@@ -1,5 +1,4 @@
 /* code128.c - Handles Code 128 and derivatives */
-
 /*
     libzint - the open source barcode library
     Copyright (C) 2008-2022 Robin Stuart <rstuart114@gmail.com>
@@ -30,12 +29,10 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
+/* SPDX-License-Identifier: BSD-3-Clause */
 
-#include <stdio.h>
-#ifdef _MSC_VER
-#include <malloc.h>
-#endif
 #include <assert.h>
+#include <stdio.h>
 #include "common.h"
 #include "code128.h"
 #include "gs1.h"
@@ -140,12 +137,12 @@ INTERNAL void c128_dxsmooth(int list[2][C128_MAX], int *indexliste) {
         if (i != 0) {
             last = list[1][i - 1];
         } else {
-            last = FALSE;
+            last = 0;
         }
         if (i != *(indexliste) - 1) {
             next = list[1][i + 1];
         } else {
-            next = FALSE;
+            next = 0;
         }
 
         if (i == 0) { /* first block */
@@ -341,8 +338,7 @@ INTERNAL void c128_put_in_set(int list[2][C128_MAX], const int indexliste, char
 }
 
 /* Treats source as ISO 8859-1 and copies into symbol->text, converting to UTF-8. Returns length of symbol->text */
-STATIC_UNLESS_ZINT_TEST int c128_hrt_cpy_iso8859_1(struct zint_symbol *symbol, const unsigned char source[],
-                            const int length) {
+static int c128_hrt_cpy_iso8859_1(struct zint_symbol *symbol, const unsigned char source[], const int length) {
     int i, j;
 
     for (i = 0, j = 0; i < length && j < (int) sizeof(symbol->text); i++) {
@@ -374,6 +370,12 @@ STATIC_UNLESS_ZINT_TEST int c128_hrt_cpy_iso8859_1(struct zint_symbol *symbol, c
     return j;
 }
 
+#ifdef ZINT_TEST /* Wrapper for direct testing */
+INTERNAL int c128_hrt_cpy_iso8859_1_test(struct zint_symbol *symbol, const unsigned char source[], const int length) {
+	return c128_hrt_cpy_iso8859_1(symbol, source, length);
+}
+#endif
+
 /* Handle Code 128, 128B and HIBC 128 */
 INTERNAL int code128(struct zint_symbol *symbol, unsigned char source[], int length) {
     int i, j, k, values[C128_MAX] = {0}, bar_characters, read, total_sum;
@@ -740,11 +742,7 @@ INTERNAL int gs1_128_cc(struct zint_symbol *symbol, unsigned char source[], int
     char *d = dest;
     int separator_row, linkage_flag;
     int reduced_length;
-#ifndef _MSC_VER
-    unsigned char reduced[length + 1];
-#else
-    unsigned char *reduced = (unsigned char *) _alloca(length + 1);
-#endif
+    unsigned char *reduced = (unsigned char *) z_alloca(length + 1);
 
     linkage_flag = 0;
 
@@ -1064,7 +1062,7 @@ INTERNAL int ean14(struct zint_symbol *symbol, unsigned char source[], int lengt
     return error_number;
 }
 
-/* DPD (Deutsher Paket Dienst) Code */
+/* DPD (Deutscher Paketdienst) Code */
 /* Specification at ftp://dpd.at/Datenspezifikationen/EN/gbs_V4.0.2_hauptdokument.pdf
  * or https://docplayer.net/33728877-Dpd-parcel-label-specification.html */
 INTERNAL int dpd(struct zint_symbol *symbol, unsigned char source[], int length) {
@@ -1072,7 +1070,7 @@ INTERNAL int dpd(struct zint_symbol *symbol, unsigned char source[], int length)
     int i, p;
     unsigned char identifier;
     const int mod = 36;
-    int cd; // Check digit
+    int cd; /* Check digit */
 
     if (length != 28) {
         strcpy(symbol->errtxt, "349: DPD input wrong length (28 characters required)");
