@@ -1,8 +1,7 @@
 /* composite.c - Handles GS1 Composite Symbols */
-
 /*
     libzint - the open source barcode library
-    Copyright (C) 2008 - 2021 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2008-2022 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -29,7 +28,7 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
-/* vim: set ts=4 sw=4 et : */
+/* SPDX-License-Identifier: BSD-3-Clause */
 
 /* The functions "getBit", "init928" and "encode928" are copyright BSI and are
    released with permission under the following terms:
@@ -49,12 +48,9 @@
    The date of publication for these functions is 31 May 2006
  */
 
-#include <stdio.h>
 #include <assert.h>
 #include <math.h>
-#ifdef _MSC_VER
-#include <malloc.h>
-#endif
+#include <stdio.h>
 #include "common.h"
 #include "pdf417.h"
 #include "gs1.h"
@@ -304,11 +300,7 @@ static void cc_a(struct zint_symbol *symbol, const char source[], const int cc_w
 static void cc_b(struct zint_symbol *symbol, const char source[], const int cc_width) {
     const int length = (int) strlen(source) / 8;
     int i;
-#ifndef _MSC_VER
-    unsigned char data_string[length + 3];
-#else
-    unsigned char *data_string = (unsigned char *) _alloca(length + 3);
-#endif
+    unsigned char *data_string = (unsigned char *) z_alloca(length + 3);
     int chainemc[180], mclength;
     int k, j, p, longueur, mccorrection[50] = {0}, offset;
     int total;
@@ -336,7 +328,7 @@ static void cc_b(struct zint_symbol *symbol, const char source[], const int cc_w
     chainemc[mclength] = 920;
     mclength++;
 
-    pdf_byteprocess(chainemc, &mclength, data_string, 0, length, debug_print);
+    pdf_byteprocess(chainemc, &mclength, data_string, 0, length, 0, debug_print);
 
     /* Now figure out which variant of the symbol to use and load values accordingly */
 
@@ -529,11 +521,7 @@ static void cc_b(struct zint_symbol *symbol, const char source[], const int cc_w
 static void cc_c(struct zint_symbol *symbol, const char source[], const int cc_width, const int ecc_level) {
     const int length = (int) strlen(source) / 8;
     int i, p;
-#ifndef _MSC_VER
-    unsigned char data_string[length + 4];
-#else
-    unsigned char *data_string = (unsigned char *) _alloca(length + 4);
-#endif
+    unsigned char *data_string = (unsigned char *) z_alloca(length + 4);
     int chainemc[1000], mclength, k;
     int offset, longueur, loop, total, j, mccorrection[520] = {0};
     int c1, c2, c3, dummy[35];
@@ -559,7 +547,7 @@ static void cc_c(struct zint_symbol *symbol, const char source[], const int cc_w
     chainemc[mclength] = 920; /* CC-C identifier */
     mclength++;
 
-    pdf_byteprocess(chainemc, &mclength, data_string, 0, length, debug_print);
+    pdf_byteprocess(chainemc, &mclength, data_string, 0, length, 0, debug_print);
 
     chainemc[0] = mclength;
 
@@ -872,11 +860,7 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
     int ai90_mode, remainder;
     char last_digit = '\0';
     int mode;
-#ifndef _MSC_VER
-    char general_field[source_len + 1];
-#else
-    char *general_field = (char *) _alloca(source_len + 1);
-#endif
+    char *general_field = (char *) z_alloca(source_len + 1);
     int target_bitsize;
     int bp = 0;
     const int debug_print = symbol->debug & ZINT_DEBUG_PRINT;
@@ -951,11 +935,7 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
 
     } else if (encoding_method == 3) {
         /* Encodation Method field of "11" - AI 90 */
-#ifndef _MSC_VER
-        char ninety[source_len + 1];
-#else
-        char *ninety = (char *) _alloca(source_len + 1);
-#endif
+        char *ninety = (char *) z_alloca(source_len + 1);
         int ninety_len, alpha, alphanum, numeric, test1, test2, test3;
 
         /* "This encodation method may be used if an element string with an AI
@@ -980,10 +960,10 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
 
         for (i = 0; i < ninety_len; i++) {
 
-            if ((ninety[i] >= 'A') && (ninety[i] <= 'Z')) {
+            if (z_isupper(ninety[i])) {
                 /* Character is alphabetic */
                 alpha += 1;
-            } else if ((ninety[i] >= '0') && (ninety[i] <= '9')) {
+            } else if (z_isdigit(ninety[i])) {
                 /* Character is numeric */
                 numeric += 1;
             } else {
@@ -994,15 +974,16 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
         /* must start with 0, 1, 2 or 3 digits followed by an uppercase character */
         test1 = -1;
         for (i = 3; i >= 0; i--) {
-            if ((ninety[i] >= 'A') && (ninety[i] <= 'Z')) {
+            if (z_isupper(ninety[i])) {
                 test1 = i;
             }
         }
 
         test2 = 0;
         for (i = 0; i < test1; i++) {
-            if (!((ninety[i] >= '0') && (ninety[i] <= '9'))) {
+            if (!z_isdigit(ninety[i])) {
                 test2 = 1;
+                break;
             }
         }
 
@@ -1109,10 +1090,10 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
             if (ai90_mode == 2) {
                 /* Alpha encodation (section 5.3.3) */
                 do {
-                    if ((source[read_posn] >= 'A') && (source[read_posn] <= 'Z')) {
+                    if (z_isupper(source[read_posn])) {
                         bp = bin_append_posn(source[read_posn] - 65, 5, binary_string, bp);
 
-                    } else if ((source[read_posn] >= '0') && (source[read_posn] <= '9')) {
+                    } else if (z_isdigit(source[read_posn])) {
                         bp = bin_append_posn(source[read_posn] + 4, 6, binary_string, bp);
 
                     } else if (source[read_posn] == '[') {
@@ -1253,20 +1234,21 @@ static int cc_binary_string(struct zint_symbol *symbol, const unsigned char sour
     return 0;
 }
 
-static int linear_dummy_run(int input_mode, unsigned char *source, const int length, char *errtxt) {
-    struct zint_symbol *dummy;
+/* Calculate the width of the linear part (primary) */
+static int linear_dummy_run(int input_mode, unsigned char *source, const int length, const int debug, char *errtxt) {
+    struct zint_symbol dummy = {0};
     int error_number;
     int linear_width;
 
-    dummy = ZBarcode_Create();
-    dummy->symbology = BARCODE_GS1_128_CC;
-    dummy->input_mode = input_mode;
-    error_number = gs1_128_cc(dummy, source, length, 3 /*cc_mode*/, 0 /*cc_rows*/);
-    linear_width = dummy->width;
-    if (error_number >= ZINT_ERROR) {
-        strcpy(errtxt, dummy->errtxt);
+    dummy.symbology = BARCODE_GS1_128_CC;
+    dummy.option_1 = -1;
+    dummy.input_mode = input_mode;
+    dummy.debug = debug;
+    error_number = gs1_128_cc(&dummy, source, length, 3 /*cc_mode*/, 0 /*cc_rows*/);
+    linear_width = dummy.width;
+    if (error_number >= ZINT_ERROR || (debug & ZINT_DEBUG_TEST)) {
+        strcpy(errtxt, dummy.errtxt);
     }
-    ZBarcode_Delete(dummy);
 
     if (error_number >= ZINT_ERROR) {
         return 0;
@@ -1281,11 +1263,7 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
     int j, i, k;
     /* Allow for 8 bits + 5-bit latch per char + 1000 bits overhead/padding */
     const unsigned int bs = 13 * length + 1000 + 1;
-#ifndef _MSC_VER
-    char binary_string[bs];
-#else
-    char *binary_string = (char *) _alloca(bs);
-#endif
+    char *binary_string = (char *) z_alloca(bs);
     unsigned int pri_len;
     struct zint_symbol *linear;
     int top_shift, bottom_shift;
@@ -1298,8 +1276,8 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
     error_number = 0;
     pri_len = (int) strlen(symbol->primary);
     if (pri_len == 0) {
-        strcpy(symbol->errtxt, "445: No primary (linear) message in 2D composite");
-        return ZINT_ERROR_INVALID_OPTION;
+        strcpy(symbol->errtxt, "445: No primary (linear) message");
+        return ZINT_ERROR_INVALID_OPTION; /* TODO: change to more appropiate ZINT_ERROR_INVALID_DATA */
     }
 
     if (length > 2990) {
@@ -1317,7 +1295,7 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
     if (symbol->symbology == BARCODE_GS1_128_CC) {
         /* Do a test run of encoding the linear component to establish its width */
         linear_width = linear_dummy_run(symbol->input_mode, (unsigned char *) symbol->primary, pri_len,
-                                        symbol->errtxt);
+                                        symbol->debug, symbol->errtxt);
         if (linear_width == 0) {
             if (strlen(symbol->errtxt) + strlen(in_linear_comp) < sizeof(symbol->errtxt)) {
                 strcat(symbol->errtxt, in_linear_comp);
@@ -1636,3 +1614,5 @@ INTERNAL int composite(struct zint_symbol *symbol, unsigned char source[], int l
 
     return error_number ? error_number : warn_number;
 }
+
+/* vim: set ts=4 sw=4 et : */
