@@ -1,5 +1,4 @@
 /* dmatrix.c Handles Data Matrix ECC 200 symbols */
-
 /*
     libzint - the open source barcode library
     Copyright (C) 2009-2022 Robin Stuart <rstuart114@gmail.com>
@@ -37,14 +36,11 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
+/* SPDX-License-Identifier: BSD-3-Clause */
 
-#include <stdio.h>
 #include <assert.h>
 #include <limits.h>
-#include <math.h>
-#ifdef _MSC_VER
-#include <malloc.h>
-#endif
+#include <stdio.h>
 #include "common.h"
 #include "reedsol.h"
 #include "dmatrix.h"
@@ -59,14 +55,14 @@ static void dm_placementbit(int *array, const int NR, const int NC, int r, int c
         c += NC;
         r += 4 - ((NC + 4) % 8);
     }
-    // Necessary for DMRE (ISO/IEC 21471:2020 Annex E)
+    /* Necessary for DMRE (ISO/IEC 21471:2020 Annex E) */
     if (r >= NR) {
         r -= NR;
     }
-    // Check index limits
+    /* Check index limits */
     assert(r < NR);
     assert(c < NC);
-    // Check double-assignment
+    /* Check double-assignment */
     assert(0 == array[r * NC + c]);
     array[r * NC + c] = (p << 3) + b;
 }
@@ -130,12 +126,12 @@ static void dm_placementcornerD(int *array, const int NR, const int NC, const in
 /* Annex F placement algorithm main function */
 static void dm_placement(int *array, const int NR, const int NC) {
     int r, c, p;
-    // start
+    /* start */
     p = 1;
     r = 4;
     c = 0;
     do {
-        // check corner
+        /* check corner */
         if (r == NR && !c)
             dm_placementcornerA(array, NR, NC, p++);
         if (r == NR - 2 && !c && NC % 4)
@@ -144,7 +140,7 @@ static void dm_placement(int *array, const int NR, const int NC) {
             dm_placementcornerC(array, NR, NC, p++);
         if (r == NR + 4 && c == 2 && !(NC % 8))
             dm_placementcornerD(array, NR, NC, p++);
-        // up/right
+        /* up/right */
         do {
             if (r < NR && c >= 0 && !array[r * NC + c])
                 dm_placementblock(array, NR, NC, r, c, p++);
@@ -153,7 +149,7 @@ static void dm_placement(int *array, const int NR, const int NC) {
         } while (r >= 0 && c < NC);
         r++;
         c += 3;
-        // down/left
+        /* down/left */
         do {
             if (r >= 0 && c < NC && !array[r * NC + c])
                 dm_placementblock(array, NR, NC, r, c, p++);
@@ -163,7 +159,7 @@ static void dm_placement(int *array, const int NR, const int NC) {
         r += 3;
         c++;
     } while (r < NR || c < NC);
-    // unfilled corner
+    /* unfilled corner */
     if (!array[NR * NC - 1])
         array[NR * NC - 1] = array[NR * NC - NC - 2] = 1;
 }
@@ -183,7 +179,7 @@ static void dm_ecc(unsigned char *binary, const int bytes, const int datablock,
         for (n = b; n < bytes; n += blocks)
             buf[p++] = binary[n];
         rs_encode(&rs, p, buf, ecc);
-        p = rsblock - 1; // comes back reversed
+        p = rsblock - 1; /* comes back reversed */
         for (n = b; n < rsblocks; n += blocks) {
             if (skew) {
                 /* Rotate ecc data to make 144x144 size symbols acceptable */
@@ -206,7 +202,7 @@ static int dm_isc40(const unsigned char input) {
     if (input <= '9') {
         return input >= '0' || input == ' ';
     }
-    return input >= 'A' && input <= 'Z';
+    return z_isupper(input);
 }
 
 /* Is basic (non-shifted) TEXT? */
@@ -214,7 +210,7 @@ static int dm_istext(const unsigned char input) {
     if (input <= '9') {
         return input >= '0' || input == ' ';
     }
-    return input >= 'a' && input <= 'z';
+    return z_islower(input);
 }
 
 /* Is basic (non-shifted) C40/TEXT? */
@@ -327,66 +323,66 @@ static int dm_look_ahead_test(const unsigned char source[], const int length, co
         const int is_extended = c & 0x80;
 
         /* ascii ... step (l) */
-        if ((c <= '9') && (c >= '0')) {
-            ascii_count += DM_MULT_1_DIV_2; // (l)(1)
+        if (z_isdigit(c)) {
+            ascii_count += DM_MULT_1_DIV_2; /* (l)(1) */
         } else {
             if (is_extended) {
-                ascii_count = DM_MULT_CEIL(ascii_count) + DM_MULT_2; // (l)(2)
+                ascii_count = DM_MULT_CEIL(ascii_count) + DM_MULT_2; /* (l)(2) */
             } else {
-                ascii_count = DM_MULT_CEIL(ascii_count) + DM_MULT_1; // (l)(3)
+                ascii_count = DM_MULT_CEIL(ascii_count) + DM_MULT_1; /* (l)(3) */
             }
         }
 
         /* c40 ... step (m) */
         if (dm_isc40(c)) {
-            c40_count += DM_MULT_2_DIV_3; // (m)(1)
+            c40_count += DM_MULT_2_DIV_3; /* (m)(1) */
         } else {
             if (is_extended) {
-                c40_count += DM_MULT_8_DIV_3; // (m)(2)
+                c40_count += DM_MULT_8_DIV_3; /* (m)(2) */
             } else {
-                c40_count += DM_MULT_4_DIV_3; // (m)(3)
+                c40_count += DM_MULT_4_DIV_3; /* (m)(3) */
             }
         }
 
         /* text ... step (n) */
         if (dm_istext(c)) {
-            text_count += DM_MULT_2_DIV_3; // (n)(1)
+            text_count += DM_MULT_2_DIV_3; /* (n)(1) */
         } else {
             if (is_extended) {
-                text_count += DM_MULT_8_DIV_3; // (n)(2)
+                text_count += DM_MULT_8_DIV_3; /* (n)(2) */
             } else {
-                text_count += DM_MULT_4_DIV_3; // (n)(3)
+                text_count += DM_MULT_4_DIV_3; /* (n)(3) */
             }
         }
 
         /* x12 ... step (o) */
         if (dm_isX12(c)) {
-            x12_count += DM_MULT_2_DIV_3; // (o)(1)
+            x12_count += DM_MULT_2_DIV_3; /* (o)(1) */
         } else {
             if (is_extended) {
-                x12_count += DM_MULT_13_DIV_3; // (o)(2)
+                x12_count += DM_MULT_13_DIV_3; /* (o)(2) */
             } else {
-                x12_count += DM_MULT_10_DIV_3; // (o)(3)
+                x12_count += DM_MULT_10_DIV_3; /* (o)(3) */
             }
         }
 
         /* edifact ... step (p) */
         if (dm_isedifact(c, gs1)) {
-            edf_count += DM_MULT_3_DIV_4; // (p)(1)
+            edf_count += DM_MULT_3_DIV_4; /* (p)(1) */
         } else {
             if (is_extended) {
-                edf_count += DM_MULT_17_DIV_4; // (p)(2)
+                edf_count += DM_MULT_17_DIV_4; /* (p)(2) */
             } else {
-                edf_count += DM_MULT_13_DIV_4; // (p)(3)
+                edf_count += DM_MULT_13_DIV_4; /* (p)(3) */
             }
         }
 
         /* base 256 ... step (q) */
         if ((gs1 == 1) && (c == '[')) {
             /* FNC1 separator */
-            b256_count += DM_MULT_4; // (q)(1)
+            b256_count += DM_MULT_4; /* (q)(1) */
         } else {
-            b256_count += DM_MULT_1; // (q)(2)
+            b256_count += DM_MULT_1; /* (q)(2) */
         }
 
         if (sp >= position + 3) {
@@ -687,7 +683,7 @@ static int dm_switch_mode(const int next_mode, unsigned char target[], int tp, i
 
 #define DM_NUM_MODES        6
 
-static char *dm_smodes[] = { "?", "ASCII", "C40", "TEXT", "X12", "EDF", "B256" };
+static const char *dm_smodes[] = { "?", "ASCII", "C40", "TEXT", "X12", "EDF", "B256" };
 
 /* The size of this structure could be significantly reduced using techniques pointed out by Alex Geller,
    but not done currently to avoid the processing overhead */
@@ -720,26 +716,22 @@ static int dm_last_ascii(const unsigned char source[], const int length, const i
         if ((source[from] & 0x80) || (source[from + 1] & 0x80)) {
             return 0;
         }
-        if (source[from] <= '9' && source[from] >= '0' && source[from + 1] <= '9' && source[from + 1] >= '0') {
+        if (z_isdigit(source[from]) && z_isdigit(source[from + 1])) {
             return 1;
         }
         return 2;
     }
     if (length - from == 3) {
-        if (source[from] <= '9' && source[from] >= '0'
-                && source[from + 1] <= '9' && source[from + 1] >= '0' && !(source[from + 2] & 0x80)) {
+        if (z_isdigit(source[from]) && z_isdigit(source[from + 1]) && !(source[from + 2] & 0x80)) {
             return 2;
         }
-        if (source[from + 1] <= '9' && source[from + 1] >= '0'
-                && source[from + 2] <= '9' && source[from + 2] >= '0' && !(source[from] & 0x80)) {
+        if (z_isdigit(source[from + 1]) && z_isdigit(source[from + 2]) && !(source[from] & 0x80)) {
             return 2;
         }
         return 0;
     }
-    if (source[from] <= '9' && source[from] >= '0'
-            && source[from + 1] <= '9' && source[from + 1] >= '0'
-            && source[from + 2] <= '9' && source[from + 2] >= '0'
-            && source[from + 3] <= '9' && source[from + 3] >= '0') {
+    if (z_isdigit(source[from]) && z_isdigit(source[from + 1]) && z_isdigit(source[from + 2])
+            && z_isdigit(source[from + 3])) {
         return 2;
     }
     return 0;
@@ -766,7 +758,7 @@ static int dm_getEndMode(struct zint_symbol *symbol, const unsigned char *source
     return mode;
 }
 
-//#define DM_TRACE
+/*#define DM_TRACE*/
 #include "dmatrix_trace.h"
 
 /* Return number of C40/TEXT codewords needed to encode characters in full batches of 3 (or less if EOD).
@@ -938,8 +930,7 @@ static void dm_addEdges(struct zint_symbol *symbol, const unsigned char source[]
 
         static const int c40text_modes[] = { DM_C40, DM_TEXT };
 
-        if (source[from] <= '9' && source[from] >= '0' && from + 1 < length
-                && source[from + 1] <= '9' && source[from + 1] >= '0') {
+        if (z_isdigit(source[from]) && from + 1 < length && z_isdigit(source[from + 1])) {
             dm_addEdge(symbol, source, length, edges, DM_ASCII, from, 2, previous, 0);
             /* If ASCII vertex, don't bother adding other edges as this will be optimal; suggested by Alex Geller */
             if (previous && previous->mode == DM_ASCII) {
@@ -1060,11 +1051,7 @@ static int dm_minimalenc(struct zint_symbol *symbol, const unsigned char source[
     int current_mode = *p_current_mode;
     int last_ascii, symbols_left;
     int i;
-#ifndef _MSC_VER
-    char modes[length];
-#else
-    char *modes = (char *) _alloca(length);
-#endif
+    char *modes = (char *) z_alloca(length);
 
     assert(length <= 10921); /* Can only handle (10921 + 1) * 6 = 65532 < 65536 (2*16) due to sizeof(previous) */
 
@@ -1190,9 +1177,9 @@ static int dm_minimalenc(struct zint_symbol *symbol, const unsigned char source[
             static const char x12_nonalphanum_chars[] = "\015*> ";
             int value = 0;
 
-            if ((source[sp] >= '0') && (source[sp] <= '9')) {
+            if (z_isdigit(source[sp])) {
                 value = (source[sp] - '0') + 4;
-            } else if ((source[sp] >= 'A') && (source[sp] <= 'Z')) {
+            } else if (z_isupper(source[sp])) {
                 value = (source[sp] - 'A') + 14;
             } else {
                 value = posn(x12_nonalphanum_chars, source[sp]);
@@ -1209,7 +1196,7 @@ static int dm_minimalenc(struct zint_symbol *symbol, const unsigned char source[
 
             int value = source[sp];
 
-            if (value >= 64) { // '@'
+            if (value >= 64) { /* '@' */
                 value -= 64;
             }
 
@@ -1378,9 +1365,9 @@ static int dm_isoenc(struct zint_symbol *symbol, const unsigned char source[], c
                 static const char x12_nonalphanum_chars[] = "\015*> ";
                 int value = 0;
 
-                if ((source[sp] <= '9') && (source[sp] >= '0')) {
+                if (z_isdigit(source[sp])) {
                     value = (source[sp] - '0') + 4;
-                } else if ((source[sp] >= 'A') && (source[sp] <= 'Z')) {
+                } else if (z_isupper(source[sp])) {
                     value = (source[sp] - 'A') + 14;
                 } else {
                     value = posn(x12_nonalphanum_chars, source[sp]);
@@ -1417,7 +1404,7 @@ static int dm_isoenc(struct zint_symbol *symbol, const unsigned char source[], c
             } else {
                 int value = source[sp];
 
-                if (value >= 64) { // '@'
+                if (value >= 64) { /* '@' */
                     value -= 64;
                 }
 
@@ -1483,8 +1470,8 @@ static int dm_isoenc(struct zint_symbol *symbol, const unsigned char source[], c
 
 /* Encodes data using ASCII, C40, Text, X12, EDIFACT or Base 256 modes as appropriate
    Supports encoding FNC1 in supporting systems */
-STATIC_UNLESS_ZINT_TEST int dm_encode(struct zint_symbol *symbol, const unsigned char source[],
-            const int length, const int eci, const int gs1, unsigned char target[], int *p_tp) {
+static int dm_encode(struct zint_symbol *symbol, const unsigned char source[], const int length, const int eci,
+            const int gs1, unsigned char target[], int *p_tp) {
     int sp = 0;
     int tp = *p_tp;
     int current_mode = DM_ASCII;
@@ -1535,20 +1522,20 @@ STATIC_UNLESS_ZINT_TEST int dm_encode(struct zint_symbol *symbol, const unsigned
         if (debug_print) printf("%s ", current_mode == DM_C40 ? "C40" : "TEX");
         if (process_p == 0) {
             if (symbols_left > 0) {
-                target[tp++] = 254; // Unlatch
+                target[tp++] = 254; /* Unlatch */
                 if (debug_print) printf("ASC ");
             }
         } else {
             if (process_p == 2 && symbols_left == 2) {
                 /* 5.2.5.2 (b) */
-                process_buffer[process_p++] = 0; // Shift 1
+                process_buffer[process_p++] = 0; /* Shift 1 */
                 (void) dm_ctx_buffer_xfer(process_buffer, process_p, target, &tp, debug_print);
 
             } else if (process_p == 1 && symbols_left <= 2 && dm_isc40text(current_mode, source[length - 1])) {
                 /* 5.2.5.2 (c)/(d) */
                 if (symbols_left > 1) {
                     /* 5.2.5.2 (c) */
-                    target[tp++] = 254; // Unlatch and encode remaining data in ascii.
+                    target[tp++] = 254; /* Unlatch and encode remaining data in ascii. */
                     if (debug_print) printf("ASC ");
                 }
                 target[tp++] = source[length - 1] + 1;
@@ -1566,7 +1553,7 @@ STATIC_UNLESS_ZINT_TEST int dm_encode(struct zint_symbol *symbol, const unsigned
                 if (debug_print) printf("Mode %d, backtracked %d\n", current_mode, (total_cnt / 3) * 2);
                 tp -= (total_cnt / 3) * 2;
 
-                target[tp++] = 254; // Unlatch
+                target[tp++] = 254; /* Unlatch */
                 if (debug_print) printf("ASC ");
                 for (; sp < length; sp++) {
                     if (is_twodigits(source, length, sp)) {
@@ -1596,12 +1583,12 @@ STATIC_UNLESS_ZINT_TEST int dm_encode(struct zint_symbol *symbol, const unsigned
     } else if (current_mode == DM_X12) {
         if (debug_print) printf("X12 ");
         if ((symbols_left == 1) && (process_p == 1)) {
-            // Unlatch not required!
+            /* Unlatch not required! */
             target[tp++] = source[length - 1] + 1;
             if (debug_print) printf("A%02X ", target[tp - 1] - 1);
         } else {
             if (symbols_left > 0) {
-                target[tp++] = (254); // Unlatch.
+                target[tp++] = (254); /* Unlatch. */
                 if (debug_print) printf("ASC ");
             }
 
@@ -1617,7 +1604,7 @@ STATIC_UNLESS_ZINT_TEST int dm_encode(struct zint_symbol *symbol, const unsigned
 
     } else if (current_mode == DM_EDIFACT) {
         if (debug_print) printf("EDI ");
-        if (symbols_left <= 2 && process_p <= symbols_left) { // Unlatch not required!
+        if (symbols_left <= 2 && process_p <= symbols_left) { /* Unlatch not required! */
             if (process_p == 1) {
                 target[tp++] = source[length - 1] + 1;
                 if (debug_print) printf("A%02X ", target[tp - 1] - 1);
@@ -1627,7 +1614,7 @@ STATIC_UNLESS_ZINT_TEST int dm_encode(struct zint_symbol *symbol, const unsigned
                 if (debug_print) printf("A%02X A%02X ", target[tp - 2] - 1, target[tp - 1] - 1);
             }
         } else {
-            // Append edifact unlatch value (31) and empty buffer
+            /* Append edifact unlatch value (31) and empty buffer */
             if (process_p <= 3) {
                 process_buffer[process_p++] = 31;
             }
@@ -1658,6 +1645,13 @@ STATIC_UNLESS_ZINT_TEST int dm_encode(struct zint_symbol *symbol, const unsigned
     return 0;
 }
 
+#ifdef ZINT_TEST /* Wrapper for direct testing */
+INTERNAL int dm_encode_test(struct zint_symbol *symbol, const unsigned char source[], const int length, const int eci,
+            const int gs1, unsigned char target[], int *p_tp) {
+    return dm_encode(symbol, source, length, eci, gs1, target, p_tp);
+}
+#endif
+
 /* Call `dm_encode()` for each segment, dealing with Structured Append, GS1, READER_INIT and macro headers
    beforehand */
 static int dm_encode_segs(struct zint_symbol *symbol, struct zint_seg segs[], const int seg_count,
@@ -1841,7 +1835,7 @@ static int dm_ecc200(struct zint_symbol *symbol, struct zint_seg segs[], const i
 
     if (binlen > dm_matrixbytes[symbolsize]) {
         if ((symbol->option_2 >= 1) && (symbol->option_2 <= DMSIZESCOUNT)) {
-            // The symbol size was given by --ver (option_2)
+            /* The symbol size was given by --ver (option_2) */
             strcpy(symbol->errtxt, "522: Input too long for selected symbol size");
         } else {
             strcpy(symbol->errtxt, "523: Data too long to fit in symbol");
@@ -1868,7 +1862,7 @@ static int dm_ecc200(struct zint_symbol *symbol, struct zint_seg segs[], const i
         printf("\n");
     }
 
-    // ecc code
+    /* ecc code */
     if (symbolsize == INTSYMBOL144) {
         skew = 1;
     }
@@ -1884,7 +1878,7 @@ static int dm_ecc200(struct zint_symbol *symbol, struct zint_seg segs[], const i
         debug_test_codeword_dump(symbol, binary, skew ? 1558 + 620 : bytes + rsblock * (bytes / datablock));
     }
 #endif
-    { // placement
+    { /* placement */
         const int NC = W - 2 * (W / FW);
         const int NR = H - 2 * (H / FH);
         int x, y, *places;
@@ -1906,7 +1900,7 @@ static int dm_ecc200(struct zint_symbol *symbol, struct zint_seg segs[], const i
                 set_module(symbol, (H - y) - 1, x + FW - 1);
         }
 #ifdef DM_DEBUG
-        // Print position matrix as in standard
+        /* Print position matrix as in standard */
         for (y = NR - 1; y >= 0; y--) {
             for (x = 0; x < NC; x++) {
                 const int v = places[(NR - y - 1) * NC + x];
