@@ -1,5 +1,5 @@
-/*  gridmtx.c - Grid Matrix
-
+/*  gridmtx.c - Grid Matrix */
+/*
     libzint - the open source barcode library
     Copyright (C) 2009-2022 Robin Stuart <rstuart114@gmail.com>
 
@@ -28,18 +28,15 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
+/* SPDX-License-Identifier: BSD-3-Clause */
 
 /* This file implements Grid Matrix as specified in
    AIM Global Document Number AIMD014 Rev. 1.63 Revised 9 Dec 2008 */
 
 #include <stdio.h>
-#ifdef _MSC_VER
-#include <malloc.h>
-#endif
 #include "common.h"
 #include "reedsol.h"
 #include "gridmtx.h"
-#include "gb2312.h"
 #include "eci.h"
 
 static const char EUROPIUM[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz ";
@@ -70,7 +67,7 @@ static int gm_in_numeral(const unsigned int ddata[], const int length, const int
        block of three numeric characters) */
     for (i = in_posn, digit_cnt = 0, nondigit = 0, nondigit_posn = 0; i < length && i < in_posn + 4 && digit_cnt < 3;
             i++) {
-        if (ddata[i] >= '0' && ddata[i] <= '9') {
+        if (z_isdigit(ddata[i])) {
             digit_cnt++;
         } else if (posn(gm_numeral_nondigits, (const char) ddata[i]) != -1) {
             if (nondigit) {
@@ -164,11 +161,7 @@ static void gm_define_mode(char *mode, const unsigned int ddata[], const int len
     char cur_mode;
     unsigned int prev_costs[GM_NUM_MODES];
     unsigned int cur_costs[GM_NUM_MODES];
-#ifndef _MSC_VER
-    char char_modes[length * GM_NUM_MODES];
-#else
-    char *char_modes = (char *) _alloca(length * GM_NUM_MODES);
-#endif
+    char *char_modes = (char *) z_alloca(length * GM_NUM_MODES);
 
     /* char_modes[i * GM_NUM_MODES + j] represents the mode to encode the code point at index i such that the final
      * segment ends in mode_types[j] and the total number of bits is minimized over all possible choices */
@@ -188,11 +181,11 @@ static void gm_define_mode(char *mode, const unsigned int ddata[], const int len
         if (!double_byte) {
             space = ddata[i] == ' ';
             if (!space) {
-                numeric = ddata[i] >= '0' && ddata[i] <= '9';
+                numeric = z_isdigit(ddata[i]);
                 if (!numeric) {
-                    lower = ddata[i] >= 'a' && ddata[i] <= 'z';
+                    lower = z_islower(ddata[i]);
                     if (!lower) {
-                        upper = ddata[i] >= 'A' && ddata[i] <= 'Z';
+                        upper = z_isupper(ddata[i]);
                         if (!upper) {
                             control = ddata[i] < 0x7F; /* Exclude DEL */
                             if (control && i + 1 < length) {
@@ -201,7 +194,7 @@ static void gm_define_mode(char *mode, const unsigned int ddata[], const int len
                         }
                     }
                 } else if (i + 1 < length) {
-                    double_digit = ddata[i + 1] >= '0' && ddata[i + 1] <= '9';
+                    double_digit = z_isdigit(ddata[i + 1]);
                 }
             }
         }
@@ -345,11 +338,7 @@ static int gm_encode(unsigned int ddata[], const int length, char binary[], cons
     int byte_count = 0;
     int shift;
     int bp = *p_bp;
-#ifndef _MSC_VER
-    char mode[length];
-#else
-    char *mode = (char *) _alloca(length);
-#endif
+    char *mode = (char *) z_alloca(length);
 
     if (eci != 0) {
         /* ECI assignment according to Table 8 */
@@ -407,13 +396,13 @@ static int gm_encode(unsigned int ddata[], const int length, char binary[], cons
                     switch (p) {
                         case 1: binary[number_pad_posn] = '1';
                             binary[number_pad_posn + 1] = '0';
-                            break; // 2 pad digits
+                            break; /* 2 pad digits */
                         case 2: binary[number_pad_posn] = '0';
                             binary[number_pad_posn + 1] = '1';
-                            break; // 1 pad digits
+                            break; /* 1 pad digits */
                         case 3: binary[number_pad_posn] = '0';
                             binary[number_pad_posn + 1] = '0';
-                            break; // 0 pad digits
+                            break; /* 0 pad digits */
                     }
                     switch (next_mode) {
                         case GM_CHINESE: bp = bin_append_posn(1019, 10, binary, bp);
@@ -524,8 +513,7 @@ static int gm_encode(unsigned int ddata[], const int length, char binary[], cons
                 }
                 if (!(done)) {
                     if (sp != (length - 1)) {
-                        if (((ddata[sp] >= '0') && (ddata[sp] <= '9')) &&
-                                ((ddata[sp + 1] >= '0') && (ddata[sp + 1] <= '9'))) {
+                        if (z_isdigit(ddata[sp]) && z_isdigit(ddata[sp + 1])) {
                             /* Two digits */
                             glyph = 8033 + (10 * (ddata[sp] - '0')) + (ddata[sp + 1] - '0');
                             sp++;
@@ -562,7 +550,7 @@ static int gm_encode(unsigned int ddata[], const int length, char binary[], cons
                 numbuf[1] = '0';
                 numbuf[2] = '0';
                 do {
-                    if ((ddata[sp] >= '0') && (ddata[sp] <= '9')) {
+                    if (z_isdigit(ddata[sp])) {
                         numbuf[p] = ddata[sp];
                         p++;
                     } else if (posn(gm_numeral_nondigits, (const char) ddata[sp]) != -1) {
@@ -652,11 +640,11 @@ static int gm_encode(unsigned int ddata[], const int length, char binary[], cons
 
             case GM_MIXED:
                 shift = 1;
-                if ((ddata[sp] >= '0') && (ddata[sp] <= '9')) {
+                if (z_isdigit(ddata[sp])) {
                     shift = 0;
-                } else if ((ddata[sp] >= 'A') && (ddata[sp] <= 'Z')) {
+                } else if (z_isupper(ddata[sp])) {
                     shift = 0;
-                } else if ((ddata[sp] >= 'a') && (ddata[sp] <= 'z')) {
+                } else if (z_islower(ddata[sp])) {
                     shift = 0;
                 } else if (ddata[sp] == ' ') {
                     shift = 0;
@@ -681,7 +669,7 @@ static int gm_encode(unsigned int ddata[], const int length, char binary[], cons
 
             case GM_UPPER:
                 shift = 1;
-                if ((ddata[sp] >= 'A') && (ddata[sp] <= 'Z')) {
+                if (z_isupper(ddata[sp])) {
                     shift = 0;
                 } else if (ddata[sp] == ' ') {
                     shift = 0;
@@ -706,7 +694,7 @@ static int gm_encode(unsigned int ddata[], const int length, char binary[], cons
 
             case GM_LOWER:
                 shift = 1;
-                if ((ddata[sp] >= 'a') && (ddata[sp] <= 'z')) {
+                if (z_islower(ddata[sp])) {
                     shift = 0;
                 } else if (ddata[sp] == ' ') {
                     shift = 0;
@@ -740,13 +728,13 @@ static int gm_encode(unsigned int ddata[], const int length, char binary[], cons
         switch (p) {
             case 1: binary[number_pad_posn] = '1';
                 binary[number_pad_posn + 1] = '0';
-                break; // 2 pad digits
+                break; /* 2 pad digits */
             case 2: binary[number_pad_posn] = '0';
                 binary[number_pad_posn + 1] = '1';
-                break; // 1 pad digit
+                break; /* 1 pad digit */
             case 3: binary[number_pad_posn] = '0';
                 binary[number_pad_posn + 1] = '0';
-                break; // 0 pad digits
+                break; /* 0 pad digits */
         }
     }
 
@@ -980,14 +968,8 @@ static void gm_place_data_in_grid(unsigned char word[], char grid[], int modules
 /* Place the layer ID into each macromodule */
 static void gm_place_layer_id(char *grid, int size, int layers, int modules, int ecc_level) {
     int i, j, layer, start, stop;
-
-#ifndef _MSC_VER
-    int layerid[layers + 1];
-    int id[modules * modules];
-#else
-    int *layerid = (int *) _alloca((layers + 1) * sizeof(int));
-    int *id = (int *) _alloca((modules * modules) * sizeof(int));
-#endif
+    int *layerid = (int *) z_alloca(sizeof(int) * (layers + 1));
+    int *id = (int *) z_alloca(sizeof(int) * (modules * modules));
 
     /* Calculate Layer IDs */
     for (i = 0; i <= layers; i++) {
@@ -1046,15 +1028,9 @@ INTERNAL int gridmatrix(struct zint_symbol *symbol, struct zint_seg segs[], cons
     int bin_len;
     const int debug_print = symbol->debug & ZINT_DEBUG_PRINT;
     const int eci_length_segs = get_eci_length_segs(segs, seg_count);
-
-#ifndef _MSC_VER
-    struct zint_seg local_segs[seg_count];
-    unsigned int ddata[eci_length_segs];
-#else
-    struct zint_seg *local_segs = (struct zint_seg *) _alloca(sizeof(struct zint_seg) * seg_count);
-    unsigned int *ddata = (unsigned int *) _alloca(sizeof(unsigned int) * eci_length_segs);
+    struct zint_seg *local_segs = (struct zint_seg *) z_alloca(sizeof(struct zint_seg) * seg_count);
+    unsigned int *ddata = (unsigned int *) z_alloca(sizeof(unsigned int) * eci_length_segs);
     char *grid;
-#endif
 
     segs_cpy(symbol, segs, seg_count, local_segs); /* Shallow copy (needed to set default ECIs & protect lengths) */
 
@@ -1225,12 +1201,7 @@ INTERNAL int gridmatrix(struct zint_symbol *symbol, struct zint_seg segs[], cons
     modules = 1 + (layers * 2);
     size_squared = size * size;
 
-#ifndef _MSC_VER
-    char grid[size_squared];
-#else
-    grid = (char *) _alloca(size_squared);
-#endif
-
+    grid = (char *) z_alloca(size_squared);
     memset(grid, '0', size_squared);
 
     gm_place_data_in_grid(word, grid, modules, size);
