@@ -1,8 +1,7 @@
 /* gs1.c - Verifies GS1 data */
-
 /*
     libzint - the open source barcode library
-    Copyright (C) 2009 - 2021 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2009-2022 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -29,12 +28,9 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
-/* vim: set ts=4 sw=4 et : */
+/* SPDX-License-Identifier: BSD-3-Clause */
 
 #include <stdio.h>
-#ifdef _MSC_VER
-#include <malloc.h>
-#endif
 #include "common.h"
 #include "gs1.h"
 
@@ -55,7 +51,7 @@ static int numeric(const unsigned char *data, int data_len, int offset, int min,
         const unsigned char *const de = d + (data_len > max ? max : data_len);
 
         for (; d < de; d++) {
-            if (*d < '0' || *d > '9') {
+            if (!z_isdigit(*d)) {
                 *p_err_no = 3;
                 *p_err_posn = d - data + 1;
                 sprintf(err_msg, "Non-numeric character '%c'", *d);
@@ -233,10 +229,10 @@ static int key(const unsigned char *data, int data_len, int offset, int min, int
     if (!length_only && data_len) {
         data += offset;
 
-        if (data[0] < '0' || data[0] > '9' || data[1] < '0' || data[1] > '9') {
+        if (!z_isdigit(data[0]) || !z_isdigit(data[1])) {
             *p_err_no = 3;
-            *p_err_posn = offset + (data[0] < '0' || data[0] > '9' ? 0 : 1) + 1;
-            sprintf(err_msg, "Non-numeric company prefix '%c'", data[0] < '0' || data[0] > '9' ? data[0] : data[1]);
+            *p_err_posn = offset + z_isdigit(data[0]) + 1;
+            sprintf(err_msg, "Non-numeric company prefix '%c'", data[z_isdigit(data[0])]);
             return 0;
         }
     }
@@ -750,7 +746,7 @@ static int iban(const unsigned char *data, int data_len, int offset, int min, in
         int checksum = 0;
         int given_checksum;
 
-        if (d[0] < 'A' || d[0] > 'Z' || d[1] < 'A' || d[1] > 'Z') { /* 1st 2 chars alphabetic country code */
+        if (!z_isupper(d[0]) || !z_isupper(d[1])) { /* 1st 2 chars alphabetic country code */
             *p_err_no = 3;
             *p_err_posn = d - data + 1;
             sprintf(err_msg, "Non-alphabetic IBAN country code '%.2s'", d);
@@ -763,7 +759,7 @@ static int iban(const unsigned char *data, int data_len, int offset, int min, in
             return 0;
         }
         d += 2;
-        if (d[0] < '0' || d[0] > '9' || d[1] < '0' || d[1] > '9') { /* 2nd 2 chars numeric checksum */
+        if (!z_isdigit(d[0]) || !z_isdigit(d[1])) { /* 2nd 2 chars numeric checksum */
             *p_err_no = 3;
             *p_err_posn = d - data + 1;
             sprintf(err_msg, "Non-numeric IBAN checksum '%.2s'", d);
@@ -863,7 +859,7 @@ static const unsigned char *coupon_vli(const unsigned char *data, const int data
         }
         de = d + vli + vli_offset;
         for (; d < de; d++) {
-            if (*d < '0' || *d > '9') {
+            if (!z_isdigit(*d)) {
                 *p_err_no = 3;
                 *p_err_posn = d - data + 1;
                 sprintf(err_msg, "Non-numeric %s '%c'", name, *d);
@@ -1176,6 +1172,42 @@ static int couponposoffer(const unsigned char *data, int data_len, int offset, i
     return 1;
 }
 
+/* Check WSG 84 latitude, longitude */
+static int latlong(const unsigned char *data, int data_len, int offset, int min, int max, int *p_err_no,
+            int *p_err_posn, char err_msg[50], const int length_only) {
+    (void)max;
+
+    data_len -= offset;
+
+    if (data_len < min || (data_len && data_len < 20)) {
+        return 0;
+    }
+
+    if (!length_only && data_len) {
+        const unsigned char *d = data + offset;
+        const unsigned char *const de = d + (data_len > max ? max : data_len);
+        uint64_t lat = 0, lng = 0;
+
+        for (; d < de; d++) {
+            if (de - d > 10) {
+                lat *= 10;
+                lat += *d - '0';
+            } else {
+                lng *= 10;
+                lng += *d - '0';
+            }
+        }
+        if (lat > 1800000000 || lng > 3600000000) {
+            *p_err_no = 3;
+            *p_err_posn = d - 1 - data + 1 - 10 * (lat > 1800000000);
+            sprintf(err_msg, "Invalid %s", lat > 1800000000 ? "latitude" : "longitude");
+            return 0;
+        }
+    }
+
+    return 1;
+}
+
 /* Generated by "php backend/tools/gen_gs1_linter.php > backend/gs1_lint.h" */
 #include "gs1_lint.h"
 
@@ -1190,14 +1222,10 @@ INTERNAL int gs1_verify(struct zint_symbol *symbol, const unsigned char source[]
     char obracket = symbol->input_mode & GS1PARENS_MODE ? '(' : '[';
     char cbracket = symbol->input_mode & GS1PARENS_MODE ? ')' : ']';
     int ai_max = chr_cnt(source, src_len, obracket) + 1; /* Plus 1 so non-zero */
-#ifndef _MSC_VER
-    int ai_value[ai_max], ai_location[ai_max], data_location[ai_max], data_length[ai_max];
-#else
-    int *ai_value = (int *) _alloca(ai_max * sizeof(int));
-    int *ai_location = (int *) _alloca(ai_max * sizeof(int));
-    int *data_location = (int *) _alloca(ai_max * sizeof(int));
-    int *data_length = (int *) _alloca(ai_max * sizeof(int));
-#endif
+    int *ai_value = (int *) z_alloca(sizeof(int) * ai_max);
+    int *ai_location = (int *) z_alloca(sizeof(int) * ai_max);
+    int *data_location = (int *) z_alloca(sizeof(int) * ai_max);
+    int *data_length = (int *) z_alloca(sizeof(int) * ai_max);
 
     /* Detect extended ASCII characters */
     for (i = 0; i < src_len; i++) {
@@ -1256,7 +1284,7 @@ INTERNAL int gs1_verify(struct zint_symbol *symbol, const unsigned char source[]
             ai_latch = 0;
         } else if (ai_latch) {
             ai_length++;
-            if ((source[i] < '0') || (source[i] > '9')) {
+            if (!z_isdigit(source[i])) {
                 ai_nonnumeric = 1;
             }
         }
@@ -1328,8 +1356,8 @@ INTERNAL int gs1_verify(struct zint_symbol *symbol, const unsigned char source[]
             }
         }
 
-        // Check for valid AI values and data lengths according to GS1 General
-        // Specifications Release 21.0.1, January 2021
+        /* Check for valid AI values and data lengths according to GS1 General
+           Specifications Release 21.0.1, January 2021 */
         for (i = 0; i < ai_count; i++) {
             int err_no, err_posn;
             char err_msg[50];
@@ -1399,3 +1427,5 @@ INTERNAL char gs1_check_digit(const unsigned char source[], const int length) {
 
     return itoc((10 - (count % 10)) % 10);
 }
+
+/* vim: set ts=4 sw=4 et : */
