@@ -38,7 +38,7 @@
 #include "common.h"
 #include "large.h"
 
-#define SODIUM  "0123456789-"
+#define SODIUM_MNS_F (IS_NUM_F | IS_MNS_F) /* SODIUM "0123456789-" */
 
 /* The following lookup tables were generated using the code in Appendix C */
 
@@ -242,11 +242,11 @@ static unsigned short USPS_MSB_Math_CRC11GenerateFrameCheckSequence(unsigned cha
     return FrameCheckSequence;
 }
 
-INTERNAL int daft_set_height(struct zint_symbol *symbol, float min_height, float max_height);
+INTERNAL int daft_set_height(struct zint_symbol *symbol, const float min_height, const float max_height);
 
-INTERNAL int imail(struct zint_symbol *symbol, unsigned char source[], int length) {
+INTERNAL int usps_imail(struct zint_symbol *symbol, unsigned char source[], int length) {
     char data_pattern[200];
-    int error_number;
+    int error_number = 0;
     int i, j, read;
     char zip[35], tracker[35], temp[2];
     large_int accum;
@@ -262,10 +262,9 @@ INTERNAL int imail(struct zint_symbol *symbol, unsigned char source[], int lengt
         strcpy(symbol->errtxt, "450: Input too long (32 character maximum)");
         return ZINT_ERROR_TOO_LONG;
     }
-    error_number = is_sane(SODIUM, source, length);
-    if (error_number == ZINT_ERROR_INVALID_DATA) {
+    if (!is_sane(SODIUM_MNS_F, source, length)) {
         strcpy(symbol->errtxt, "451: Invalid character in data (digits and \"-\" only)");
-        return error_number;
+        return ZINT_ERROR_INVALID_DATA;
     }
 
     strcpy(zip, "");
@@ -434,22 +433,22 @@ INTERNAL int imail(struct zint_symbol *symbol, unsigned char source[], int lengt
         read += 2;
     }
 
-#ifdef COMPLIANT_HEIGHTS
-    /* USPS-B-3200 Section 2.3.1
-       Using bar pitch as X (1" / 43) ~ 0.023" based on 22 bars + 21 spaces per inch (bar width 0.015" - 0.025"),
-       height 0.125" - 0.165"
-       Tracker 0.048" (average of 0.039" - 0.057")
-       Ascender/descender 0.0965" (average of 0.082" - 0.111") less T = 0.0485"
-     */
-    symbol->row_height[0] = 0.0485f * 43; /* 2.0855 */
-    symbol->row_height[1] = 0.048f * 43; /* 2.064 */
-    /* Note using max X for minimum and min X for maximum */
-    error_number = daft_set_height(symbol, 0.125f * 39 /*4.875*/, 0.165f * 47 /*7.755*/);
-#else
-    symbol->row_height[0] = 3.0f;
-    symbol->row_height[1] = 2.0f;
-    daft_set_height(symbol, 0.0f, 0.0f);
-#endif
+    if (symbol->output_options & COMPLIANT_HEIGHT) {
+        /* USPS-B-3200 Section 2.3.1
+           Using bar pitch as X (1" / 43) ~ 0.023" based on 22 bars + 21 spaces per inch (bar width 0.015" - 0.025"),
+           height 0.125" - 0.165"
+           Tracker 0.048" (average of 0.039" - 0.057")
+           Ascender/descender 0.0965" (average of 0.082" - 0.111") less T = 0.0485"
+         */
+        symbol->row_height[0] = stripf(0.0485f * 43); /* 2.0855 */
+        symbol->row_height[1] = stripf(0.048f * 43); /* 2.064 */
+        /* Note using max X for minimum and min X for maximum */
+        error_number = daft_set_height(symbol, stripf(0.125f * 39) /*4.875*/, stripf(0.165f * 47) /*7.755*/);
+    } else {
+        symbol->row_height[0] = 3.0f;
+        symbol->row_height[1] = 2.0f;
+        (void) daft_set_height(symbol, 0.0f, 0.0f);
+    }
     symbol->rows = 3;
     symbol->width = read - 1;
     return error_number;
