@@ -1,8 +1,7 @@
 /* large.c - Handles binary manipulation of large numbers */
-
 /*
     libzint - the open source barcode library
-    Copyright (C) 2008 - 2021 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2008-2022 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -29,7 +28,7 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
-/* vim: set ts=4 sw=4 et : */
+/* SPDX-License-Identifier: BSD-3-Clause */
 
 /* `large_mul_u64()` and `large_div_u64()` are adapted from articles by F. W. Jacob
  *   https://www.codeproject.com/Tips/618570/UInt-Multiplication-Squaring
@@ -47,9 +46,6 @@
  */
 #include <assert.h>
 #include <stdio.h>
-#ifdef _MSC_VER
-#include <malloc.h>
-#endif
 #include "common.h"
 #include "large.h"
 
@@ -59,7 +55,7 @@
 INTERNAL void large_load_str_u64(large_int *t, const unsigned char *s, const int length) {
     uint64_t val = 0;
     const unsigned char *const se = s + length;
-    for (; s < se && *s >= '0' && *s <= '9'; s++) {
+    for (; s < se && z_isdigit(*s); s++) {
         val *= 10;
         val += *s - '0';
     }
@@ -133,7 +129,7 @@ INTERNAL void large_mul_u64(large_int *t, const uint64_t s) {
 }
 
 /* Count leading zeroes. See Hickman `r128__clz64()` */
-STATIC_UNLESS_ZINT_TEST int clz_u64(uint64_t x) {
+static int clz_u64(uint64_t x) {
    uint64_t n = 64, y;
    y = x >> 32; if (y) { n -= 32; x = y; }
    y = x >> 16; if (y) { n -= 16; x = y; }
@@ -144,7 +140,13 @@ STATIC_UNLESS_ZINT_TEST int clz_u64(uint64_t x) {
    return (int) (n - x);
 }
 
-/* Divide 128-bit dividend `t` by 64-bit divisor `v`
+#ifdef ZINT_TEST /* Wrapper for direct testing */
+INTERNAL int clz_u64_test(uint64_t x) {
+    return clz_u64(x);
+}
+#endif
+
+/* Divide 128-bit dividend `t` by 64-bit divisor `v`, returning 64-bit remainder
  * See Jacob `divmod128by128/64()` and Warren Section 9–2 (divmu64.c.txt)
  * Note digits are 32-bit parts */
 INTERNAL uint64_t large_div_u64(large_int *t, uint64_t v) {
@@ -281,11 +283,7 @@ INTERNAL void large_uint_array(const large_int *t, unsigned int *uint_array, con
 /* As `large_uint_array()` above, except output to unsigned char array */
 INTERNAL void large_uchar_array(const large_int *t, unsigned char *uchar_array, const int size, int bits) {
     int i;
-#ifndef _MSC_VER
-    unsigned int uint_array[size ? size : 1]; /* Avoid run-time warning if size is 0 */
-#else
-    unsigned int *uint_array = (unsigned int *) _alloca(sizeof(unsigned int) * (size ? size : 1));
-#endif
+    unsigned int *uint_array = (unsigned int *) z_alloca(sizeof(unsigned int) * (size ? size : 1));
 
     large_uint_array(t, uint_array, size, bits);
 
@@ -294,13 +292,6 @@ INTERNAL void large_uchar_array(const large_int *t, unsigned char *uchar_array,
     }
 }
 
-/* Output formatted large_int to stdout */
-INTERNAL void large_print(const large_int *t) {
-    char buf[35]; /* 2 (0x) + 32 (hex) + 1 */
-
-    puts(large_dump(t, buf));
-}
-
 /* Format large_int into buffer, which should be at least 35 chars in size */
 INTERNAL char *large_dump(const large_int *t, char *buf) {
     unsigned int tlo1 = (unsigned int) (large_lo(t) >> 32);
@@ -319,3 +310,12 @@ INTERNAL char *large_dump(const large_int *t, char *buf) {
     }
     return buf;
 }
+
+/* Output formatted large_int to stdout */
+INTERNAL void large_print(const large_int *t) {
+    char buf[35]; /* 2 (0x) + 32 (hex) + 1 */
+
+    puts(large_dump(t, buf));
+}
+
+/* vim: set ts=4 sw=4 et : */
