@@ -29,15 +29,11 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
 */
+/* SPDX-License-Identifier: BSD-3-Clause */
+
 #ifndef Z_LARGE_H
 #define Z_LARGE_H
 
-#ifndef _MSC_VER
-#include <stdint.h>
-#else
-#include "ms_stdint.h"
-#endif
-
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -48,7 +44,7 @@ typedef struct { uint64_t lo; uint64_t hi; } large_int;
 #define large_hi(s) ((s)->hi)
 
 /* Set 128-bit `t` from 128-bit `s` */
-#define large_load(t, s) do { (t)->lo = (s)->lo; (t)->hi = (s)->hi; } while (0)
+#define large_load(t, s) do { *(t) = *(s); } while (0)
 
 /* Set 128-bit `t` from 64-bit `s` */
 #define large_load_u64(t, s) do { (t)->lo = (s); (t)->hi = 0; } while (0)
@@ -69,8 +65,8 @@ INTERNAL void large_unset_bit(large_int *t, const int bit);
 INTERNAL void large_uint_array(const large_int *t, unsigned int *uint_array, const int size, int bits);
 INTERNAL void large_uchar_array(const large_int *t, unsigned char *uchar_array, const int size, int bits);
 
-INTERNAL void large_print(const large_int *t);
 INTERNAL char *large_dump(const large_int *t, char *buf);
+INTERNAL void large_print(const large_int *t);
 
 #ifdef __cplusplus
 }
