@@ -1,5 +1,5 @@
-/*  library.c - external functions of libzint
-
+/*  library.c - external functions of libzint */
+/*
     libzint - the open source barcode library
     Copyright (C) 2009-2022 Robin Stuart <rstuart114@gmail.com>
 
@@ -28,13 +28,11 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
+/* SPDX-License-Identifier: BSD-3-Clause */
 
 #include <assert.h>
 #include <errno.h>
 #include <limits.h>
-#ifdef _MSC_VER
-#include <malloc.h>
-#endif
 #include <stdio.h>
 #include "common.h"
 #include "eci.h"
@@ -43,7 +41,12 @@
 
 /* It's assumed that int is at least 32 bits, the following will compile-time fail if not
  * https://stackoverflow.com/a/1980056 */
-typedef int static_assert_int_at_least_32bits[CHAR_BIT != 8 || sizeof(int) < 4 ? -1 : 1];
+typedef char static_assert_int_at_least_32bits[CHAR_BIT != 8 || sizeof(int) < 4 ? -1 : 1];
+
+typedef char static_assert_uint16_is_16bits[sizeof(uint16_t) != 2 ? -1 : 1];
+typedef char static_assert_int32_is_32bits[sizeof(int32_t) != 4 ? -1 : 1];
+typedef char static_assert_uint32_is_32bits[sizeof(uint32_t) != 4 ? -1 : 1];
+typedef char static_assert_uint64_at_least_64bits[sizeof(uint64_t) < 8 ? -1 : 1];
 
 /* Create and initialize a symbol structure */
 struct zint_symbol *ZBarcode_Create(void) {
@@ -64,10 +67,10 @@ struct zint_symbol *ZBarcode_Create(void) {
     strcpy(symbol->outfile, "out.png");
 #endif
     symbol->option_1 = -1;
-    symbol->show_hrt = 1; // Show human readable text
+    symbol->show_hrt = 1; /* Show human readable text */
     symbol->fontsize = 8;
     symbol->input_mode = DATA_MODE;
-    symbol->eci = 0; // Default 0 uses ECI 3
+    symbol->eci = 0; /* Default 0 uses ECI 3 */
     symbol->dot_size = 4.0f / 5.0f;
     symbol->guard_descent = 5.0f;
     symbol->warn_level = WARN_DEFAULT;
@@ -106,7 +109,7 @@ void ZBarcode_Clear(struct zint_symbol *symbol) {
     symbol->bitmap_height = 0;
     symbol->bitmap_byte_length = 0;
 
-    // If there is a rendered version, ensure its memory is released
+    /* If there is a rendered version, ensure its memory is released */
     vector_free(symbol);
 }
 
@@ -119,12 +122,13 @@ void ZBarcode_Delete(struct zint_symbol *symbol) {
     if (symbol->alphamap != NULL)
         free(symbol->alphamap);
 
-    // If there is a rendered version, ensure its memory is released
+    /* If there is a rendered version, ensure its memory is released */
     vector_free(symbol);
 
     free(symbol);
 }
 
+/* Symbology handlers */
 INTERNAL int eanx(struct zint_symbol *symbol, unsigned char source[], int length); /* EAN system barcodes */
 INTERNAL int code39(struct zint_symbol *symbol, unsigned char source[], int length); /* Code 3 from 9 (or Code 39) */
 INTERNAL int pzn(struct zint_symbol *symbol, unsigned char source[], int length); /* Pharmazentral Nummer (PZN) */
@@ -196,12 +200,14 @@ INTERNAL int mailmark(struct zint_symbol *symbol, unsigned char source[], int le
 INTERNAL int ultra(struct zint_symbol *symbol, struct zint_seg segs[], const int seg_count); /* Ultracode */
 INTERNAL int rmqr(struct zint_symbol *symbol, struct zint_seg segs[], const int seg_count); /* rMQR */
 INTERNAL int dpd(struct zint_symbol *symbol, unsigned char source[], int length); /* DPD Code */
+INTERNAL int bc412(struct zint_symbol *symbol, unsigned char source[], int length); /* BC412 */
 
+/* Output handlers */
 INTERNAL int plot_raster(struct zint_symbol *symbol, int rotate_angle, int file_type); /* Plot to PNG/BMP/PCX */
 INTERNAL int plot_vector(struct zint_symbol *symbol, int rotate_angle, int file_type); /* Plot to EPS/EMF/SVG */
 
 /* Prefix error message with Error/Warning */
-STATIC_UNLESS_ZINT_TEST int error_tag(struct zint_symbol *symbol, int error_number, const char *error_string) {
+static int error_tag(struct zint_symbol *symbol, int error_number, const char *error_string) {
 
     if (error_number != 0) {
         static const char error_fmt[] = "Error %.93s"; /* Truncate if too long */
@@ -227,6 +233,12 @@ STATIC_UNLESS_ZINT_TEST int error_tag(struct zint_symbol *symbol, int error_numb
     return error_number;
 }
 
+#ifdef ZINT_TEST /* Wrapper for direct testing */
+INTERNAL int error_tag_test(struct zint_symbol *symbol, int error_number, const char *error_string) {
+    return error_tag(symbol, error_number, error_string);
+}
+#endif
+
 /* Output a hexadecimal representation of the rendered symbol */
 static int dump_plot(struct zint_symbol *symbol) {
     FILE *f;
@@ -286,6 +298,7 @@ static int dump_plot(struct zint_symbol *symbol) {
     return 0;
 }
 
+/* Permitted HIBC characters */
 static const char TECHNETIUM[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ-. $/+%"; /* Same as SILVER (CODE39) */
 
 /* Process health industry bar code data */
@@ -389,9 +402,9 @@ static int gs1_compliant(const int symbology) {
         case BARCODE_CODEONE:
         case BARCODE_ULTRA:
         case BARCODE_RMQR:
-        // TODO: case BARCODE_CODABLOCKF:
-        // TODO: case BARCODE_HANXIN:
-        // TODO: case BARCODE_GRIDMATRIX:
+        /* TODO: case BARCODE_CODABLOCKF: */
+        /* TODO: case BARCODE_HANXIN: */
+        /* TODO: case BARCODE_GRIDMATRIX: */
             return 1;
             break;
     }
@@ -508,7 +521,7 @@ static const void *barcode_funcs[BARCODE_LAST + 1] = {
           eanx,        NULL,        eanx,        eanx,        NULL, /*35-39*/
        postnet,        NULL,        NULL,        NULL,        NULL, /*40-44*/
           NULL,        NULL, msi_plessey,        NULL,         fim, /*45-49*/
-        code39,      pharma,         pzn,  pharma_two,        NULL, /*50-54*/
+        code39,      pharma,         pzn,  pharma_two,     postnet, /*50-54*/
         pdf417,      pdf417,    maxicode,      qrcode,        NULL, /*55-59*/
        code128,        NULL,        NULL,     auspost,        NULL, /*60-64*/
           NULL,     auspost,     auspost,     auspost,        eanx, /*65-69*/
@@ -527,11 +540,13 @@ static const void *barcode_funcs[BARCODE_LAST + 1] = {
      composite,   composite,   composite,   composite,   composite, /*130-134*/
      composite,   composite,   composite,   composite,   composite, /*135-139*/
        channel,     codeone,  gridmatrix,       upnqr,       ultra, /*140-144*/
-          rmqr,
+          rmqr,       bc412,
 };
 
 static int reduced_charset(struct zint_symbol *symbol, struct zint_seg segs[], const int seg_count);
 
+/* Main dispatch, checking for barcodes which handle ECIs/character sets themselves, otherwise calling
+   `reduced_charset()` */
 static int extended_or_reduced_charset(struct zint_symbol *symbol, struct zint_seg segs[], const int seg_count) {
     int error_number = 0;
 
@@ -560,23 +575,13 @@ static int extended_or_reduced_charset(struct zint_symbol *symbol, struct zint_s
 static int reduced_charset(struct zint_symbol *symbol, struct zint_seg segs[], const int seg_count) {
     int error_number = 0;
     int i;
-
-#ifndef _MSC_VER
-    struct zint_seg local_segs[seg_count];
-    int convertible[seg_count];
-#else
-    struct zint_seg *local_segs = (struct zint_seg *) _alloca(sizeof(struct zint_seg) * seg_count);
-    int *convertible = (int *) _alloca(sizeof(int) * seg_count);
-#endif
+    struct zint_seg *local_segs = (struct zint_seg *) z_alloca(sizeof(struct zint_seg) * seg_count);
+    int *convertible = (int *) z_alloca(sizeof(int) * seg_count);
 
     if ((symbol->input_mode & 0x07) == UNICODE_MODE && is_eci_convertible_segs(segs, seg_count, convertible)) {
         unsigned char *preprocessed;
         const int eci_length_segs = get_eci_length_segs(segs, seg_count);
-#ifndef _MSC_VER
-        unsigned char preprocessed_buf[eci_length_segs + seg_count];
-#else
-        unsigned char *preprocessed_buf = (unsigned char *) _alloca(eci_length_segs + seg_count);
-#endif
+        unsigned char *preprocessed_buf = (unsigned char *) z_alloca(eci_length_segs + seg_count);
 
         /* Prior check ensures ECI only set for those that support it */
         segs_cpy(symbol, segs, seg_count, local_segs); /* Shallow copy (needed to set default ECIs) */
@@ -620,7 +625,8 @@ static int reduced_charset(struct zint_symbol *symbol, struct zint_seg segs[], c
 #pragma GCC diagnostic pop
 #endif
 
-STATIC_UNLESS_ZINT_TEST void strip_bom(unsigned char *source, int *input_length) {
+/* Remove Unicode BOM at start of data */
+static void strip_bom(unsigned char *source, int *input_length) {
     int i;
 
     /* Note if BOM is only data then not stripped */
@@ -633,28 +639,66 @@ STATIC_UNLESS_ZINT_TEST void strip_bom(unsigned char *source, int *input_length)
     }
 }
 
-STATIC_UNLESS_ZINT_TEST int escape_char_process(struct zint_symbol *symbol, unsigned char *input_string,
-            int *length) {
-    int in_posn, out_posn;
-    int hex1, hex2;
-    int i, unicode;
-
-#ifndef _MSC_VER
-    unsigned char escaped_string[*length + 1];
-#else
-    unsigned char *escaped_string = (unsigned char *) _alloca(*length + 1);
+#ifdef ZINT_TEST /* Wrapper for direct testing */
+INTERNAL void strip_bom_test(unsigned char *source, int *input_length) {
+    strip_bom(source, input_length);
+}
 #endif
 
+/* Helper to convert base octal, decimal, hexadecimal escape sequence */
+static int esc_base(struct zint_symbol *symbol, unsigned char *input_string, int length, int in_posn, int base) {
+    int c1, c2, c3;
+    int min_len = base == 'x' ? 2 : 3;
+
+    if (in_posn + min_len > length) {
+        sprintf(symbol->errtxt, "232: Incomplete '\\%c' escape sequence in input data", base);
+        return -1;
+    }
+    c1 = ctoi(input_string[in_posn]);
+    c2 = ctoi(input_string[in_posn + 1]);
+    if (base == 'd') {
+        c3 = ctoi(input_string[in_posn + 2]);
+        if ((c1 >= 0 && c1 <= 9) && (c2 >= 0 && c2 <= 9) && (c3 >= 0 && c3 <= 9)) {
+            return c1 * 100 + c2 * 10 + c3;
+        }
+    } else if (base == 'o') {
+        c3 = ctoi(input_string[in_posn + 2]);
+        if ((c1 >= 0 && c1 <= 7) && (c2 >= 0 && c2 <= 7) && (c3 >= 0 && c3 <= 7)) {
+            return (c1 << 6) | (c2 << 3) | c3;
+        }
+    } else {
+        if ((c1 >= 0) && (c2 >= 0)) {
+            return (c1 << 4) | c2;
+        }
+    }
+
+    sprintf(symbol->errtxt, "233: Invalid character for '\\%c' escape sequence in input data (%s only)",
+            base, base == 'd' ? "decimal" : base == 'o' ? "octal" : "hexadecimal" );
+    return -1;
+}
+
+/* Helper to parse escape sequences */
+static int escape_char_process(struct zint_symbol *symbol, unsigned char *input_string, int *p_length) {
+    const int length = *p_length;
+    int in_posn, out_posn;
+    int ch;
+    int val;
+    int i;
+    unsigned long unicode;
+    unsigned char *escaped_string = (unsigned char *) z_alloca(length + 1);
+
     in_posn = 0;
     out_posn = 0;
 
     do {
         if (input_string[in_posn] == '\\') {
-            if (in_posn + 1 >= *length) {
+            if (in_posn + 1 >= length) {
                 strcpy(symbol->errtxt, "236: Incomplete escape character in input data");
                 return ZINT_ERROR_INVALID_DATA;
             }
-            switch (input_string[in_posn + 1]) {
+            ch = input_string[in_posn + 1];
+            /* NOTE: if add escape character, must also update regex in "frontend_qt/datawindow.php" */
+            switch (ch) {
                 case '0': escaped_string[out_posn] = 0x00; /* Null */
                     in_posn += 2;
                     break;
@@ -691,43 +735,47 @@ STATIC_UNLESS_ZINT_TEST int escape_char_process(struct zint_symbol *symbol, unsi
                 case 'R': escaped_string[out_posn] = 0x1e; /* Record Separator */
                     in_posn += 2;
                     break;
-                case 'x': if (in_posn + 4 > *length) {
-                        strcpy(symbol->errtxt, "232: Incomplete escape character in input data");
-                        return ZINT_ERROR_INVALID_DATA;
-                    }
-                    hex1 = ctoi(input_string[in_posn + 2]);
-                    hex2 = ctoi(input_string[in_posn + 3]);
-                    if ((hex1 >= 0) && (hex2 >= 0)) {
-                        escaped_string[out_posn] = (hex1 << 4) + hex2;
-                        in_posn += 4;
-                    } else {
-                        strcpy(symbol->errtxt, "233: Corrupt escape character in input data");
+                case 'd':
+                case 'o': /* Undocumented (as not very useful for most people) */
+                case 'x':
+                    if ((val = esc_base(symbol, input_string, length, in_posn + 2, ch)) == -1) {
                         return ZINT_ERROR_INVALID_DATA;
                     }
+                    escaped_string[out_posn] = val;
+                    in_posn += 4 + (ch != 'x');
                     break;
                 case '\\': escaped_string[out_posn] = '\\';
                     in_posn += 2;
                     break;
                 case 'u':
-                    if (in_posn + 6 > *length) {
-                        strcpy(symbol->errtxt, "209: Incomplete Unicode escape character in input data");
+                case 'U':
+                    if (in_posn + 6 > length || (ch == 'U' && in_posn + 8 > length)) {
+                        sprintf(symbol->errtxt, "209: Incomplete '\\%c' escape sequence in input data", ch);
                         return ZINT_ERROR_INVALID_DATA;
                     }
                     unicode = 0;
-                    for (i = 0; i < 4; i++) {
-                        if (ctoi(input_string[in_posn + i + 2]) == -1) {
-                            strcpy(symbol->errtxt, "211: Corrupt Unicode escape character in input data");
+                    for (i = 0; i < 6; i++) {
+                        if ((val = ctoi(input_string[in_posn + i + 2])) == -1) {
+                            sprintf(symbol->errtxt,
+                                "211: Invalid character for '\\%c' escape sequence in input data (hexadecimal only)",
+                                ch);
                             return ZINT_ERROR_INVALID_DATA;
                         }
-                        unicode = unicode << 4;
-                        unicode += ctoi(input_string[in_posn + i + 2]);
+                        unicode = (unicode << 4) | val;
+                        if (i == 3 && ch == 'u') {
+                            break;
+                        }
                     }
-                    /* Exclude reversed BOM and surrogates */
-                    if (unicode == 0xfffe || (unicode >= 0xd800 && unicode < 0xe000)) {
-                        strcpy(symbol->errtxt, "246: Invalid Unicode BMP escape character in input data");
+                    /* Exclude reversed BOM and surrogates and out-of-range */
+                    if (unicode == 0xfffe || (unicode >= 0xd800 && unicode < 0xe000) || unicode > 0x10ffff) {
+                        sprintf(symbol->errtxt, "246: Invalid value for '\\%c' escape sequence in input data", ch);
                         return ZINT_ERROR_INVALID_DATA;
                     }
                     if (unicode >= 0x800) {
+                        if (unicode >= 0x10000) {
+                            escaped_string[out_posn] = 0xf0 + (unsigned char) (unicode >> 16);
+                            out_posn++;
+                        }
                         escaped_string[out_posn] = 0xe0 + ((unicode & 0xf000) >> 12);
                         out_posn++;
                         escaped_string[out_posn] = 0x80 + ((unicode & 0x0fc0) >> 6);
@@ -740,9 +788,9 @@ STATIC_UNLESS_ZINT_TEST int escape_char_process(struct zint_symbol *symbol, unsi
                     } else {
                         escaped_string[out_posn] = unicode & 0x7f;
                     }
-                    in_posn += 6;
+                    in_posn += 6 + (ch == 'U') * 2;
                     break;
-                default: strcpy(symbol->errtxt, "234: Unrecognised escape character in input data");
+                default: sprintf(symbol->errtxt, "234: Unrecognised escape character '\\%c' in input data", ch);
                     return ZINT_ERROR_INVALID_DATA;
                     break;
             }
@@ -751,15 +799,21 @@ STATIC_UNLESS_ZINT_TEST int escape_char_process(struct zint_symbol *symbol, unsi
             in_posn++;
         }
         out_posn++;
-    } while (in_posn < *length);
+    } while (in_posn < length);
 
     memcpy(input_string, escaped_string, out_posn);
     input_string[out_posn] = '\0';
-    *length = out_posn;
+    *p_length = out_posn;
 
     return 0;
 }
 
+#ifdef ZINT_TEST /* Wrapper for direct testing */
+INTERNAL int escape_char_process_test(struct zint_symbol *symbol, unsigned char *input_string, int *length) {
+    return escape_char_process(symbol, input_string, length);
+}
+#endif
+
 /* Encode a barcode. If `length` is 0, `source` must be NUL-terminated. */
 int ZBarcode_Encode(struct zint_symbol *symbol, const unsigned char *source, int length) {
     struct zint_seg segs[1];
@@ -780,26 +834,24 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
     int have_zero_eci = 0;
     int i;
     unsigned char *local_source;
-#ifndef _MSC_VER
-    struct zint_seg local_segs[seg_count > 0 ? seg_count : 1];
-#else
     struct zint_seg *local_segs;
     unsigned char *local_sources;
-    local_segs = (struct zint_seg *) _alloca(sizeof(struct zint_seg) * (seg_count > 0 ? seg_count : 1));
-#endif
+    local_segs = (struct zint_seg *) z_alloca(sizeof(struct zint_seg) * (seg_count > 0 ? seg_count : 1));
 
     if (!symbol) return ZINT_ERROR_INVALID_DATA;
 
     if (segs == NULL) {
         return error_tag(symbol, ZINT_ERROR_INVALID_DATA, "200: Input segments NULL");
     }
-    if (seg_count <= 0) {
-        return error_tag(symbol, ZINT_ERROR_INVALID_DATA, "205: Input segment count 0");
-    }
+    /* `seg_count` zero dealt with via `total_len` zero below */
     if (seg_count > ZINT_MAX_SEG_COUNT) {
         return error_tag(symbol, ZINT_ERROR_INVALID_DATA, "771: Too many input segments (max 256)");
     }
 
+    if ((symbol->input_mode & 0x07) > 2) {
+        symbol->input_mode = DATA_MODE; /* Reset completely TODO: in future, warn/error */
+    }
+
     /* Check segment lengths */
     for (i = 0; i < seg_count; i++) {
         local_segs[i] = segs[i];
@@ -811,7 +863,18 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
             local_segs[i].length = (int) ustrlen(local_segs[i].source);
         }
         if (local_segs[i].length <= 0) {
-            sprintf(symbol->errtxt, "773: Input segment %d length zero", i);
+            if (i == 0) {
+                /* Note: should really be referencing the symbology only after the symbology check switch below */
+                if (is_composite(symbol->symbology) &&
+                        ((symbol->input_mode & 0x07) == GS1_MODE || check_force_gs1(symbol->symbology))) {
+                    strcpy(symbol->errtxt, "779: No composite data in 2D component");
+                } else {
+                    sprintf(symbol->errtxt, "778: No input data%s",
+                            supports_eci(symbol->symbology) ? " (segment 0 empty)" : "");
+                }
+            } else {
+                sprintf(symbol->errtxt, "773: Input segment %d empty", i);
+            }
             return error_tag(symbol, ZINT_ERROR_INVALID_DATA, NULL);
         }
         if (local_segs[i].length > ZINT_MAX_DATA_LEN) {
@@ -820,6 +883,10 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
         total_len += local_segs[i].length;
     }
 
+    if (total_len == 0) {
+        return error_tag(symbol, ZINT_ERROR_INVALID_DATA, "205: No input data");
+    }
+
     if (symbol->debug & ZINT_DEBUG_PRINT) {
         printf("ZBarcode_Encode_Segs: symbology: %d, input_mode: 0x%X, ECI: %d, option_1: %d, option_2: %d,"
                 " option_3: %d, scale: %g\n    output_options: 0x%X, fg: %s, bg: %s,"
@@ -888,12 +955,6 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
             symbol->symbology = BARCODE_PLESSEY;
         } else if (symbol->symbology == 48) {
             symbol->symbology = BARCODE_NVE18;
-        } else if (symbol->symbology == 54) { /* General Parcel up to tbarcode 9, Brazilian CEPNet for tbarcode 10+ */
-            warn_number = error_tag(symbol, ZINT_WARN_INVALID_OPTION, "210: General Parcel Code not supported");
-            if (warn_number >= ZINT_ERROR) {
-                return warn_number;
-            }
-            symbol->symbology = BARCODE_CODE128;
         } else if ((symbol->symbology == 59) || (symbol->symbology == 61)) {
             symbol->symbology = BARCODE_CODE128;
         } else if (symbol->symbology == 62) {
@@ -1007,17 +1068,13 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
         return error_tag(symbol, ZINT_ERROR_TOO_LONG, "770: Too many stacked symbols");
     }
 
-    if ((symbol->input_mode & 0x07) > 2) {
-        symbol->input_mode = DATA_MODE; /* Reset completely TODO: in future, warn/error */
-    }
-
     if ((symbol->input_mode & 0x07) == GS1_MODE && !gs1_compliant(symbol->symbology)) {
         return error_tag(symbol, ZINT_ERROR_INVALID_OPTION, "220: Selected symbology does not support GS1 mode");
     }
     if (seg_count > 1) {
         /* Note: GS1_MODE not currently supported when using multiple segments */
         if ((symbol->input_mode & 0x07) == GS1_MODE) {
-            return error_tag(symbol, ZINT_ERROR_INVALID_OPTION, "776: GS1_MODE not supported for multiple segments");
+            return error_tag(symbol, ZINT_ERROR_INVALID_OPTION, "776: GS1 mode not supported for multiple segments");
         }
     }
 
@@ -1029,11 +1086,7 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
         }
     }
 
-#ifndef _MSC_VER
-    unsigned char local_sources[total_len + seg_count];
-#else
-    local_sources = (unsigned char *) _alloca(total_len + seg_count);
-#endif
+    local_sources = (unsigned char *) z_alloca(total_len + seg_count);
 
     for (i = 0, local_source = local_sources; i < seg_count; i++) {
         local_segs[i].source = local_source;
@@ -1050,6 +1103,13 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
                 return error_tag(symbol, error_number, NULL);
             }
         }
+        if (symbol->primary[0]) {
+            int primary_len = (int) strlen(symbol->primary);
+            error_number = escape_char_process(symbol, (unsigned char *) symbol->primary, &primary_len);
+            if (error_number != 0) { /* Only returns errors, not warnings */
+                return error_tag(symbol, error_number, NULL);
+            }
+        }
     }
 
     if ((symbol->input_mode & 0x07) == UNICODE_MODE) {
@@ -1058,15 +1118,11 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
     }
 
     if (((symbol->input_mode & 0x07) == GS1_MODE) || (check_force_gs1(symbol->symbology))) {
-        if (gs1_compliant(symbol->symbology) == 1) {
-            // Reduce input for composite and non-forced symbologies, others (EAN128 and RSS_EXP based) will
-            // handle it themselves
+        if (gs1_compliant(symbol->symbology)) {
+            /* Reduce input for composite and non-forced symbologies, others (EAN128 and RSS_EXP based) will
+               handle it themselves */
             if (is_composite(symbol->symbology) || !check_force_gs1(symbol->symbology)) {
-#ifndef _MSC_VER
-                unsigned char reduced[local_segs[0].length + 1];
-#else
-                unsigned char *reduced = (unsigned char *) _alloca(local_segs[0].length + 1);
-#endif
+                unsigned char *reduced = (unsigned char *) z_alloca(local_segs[0].length + 1);
                 error_number = gs1_verify(symbol, local_segs[0].source, local_segs[0].length, reduced);
                 if (error_number) {
                     static const char in_2d_comp[] = " in 2D component";
@@ -1080,7 +1136,7 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
                     }
                     warn_number = error_number; /* Override any previous warning (errtxt has been overwritten) */
                 }
-                ustrcpy(local_segs[0].source, reduced); // Cannot contain NUL char
+                ustrcpy(local_segs[0].source, reduced); /* Cannot contain NUL char */
                 local_segs[0].length = (int) ustrlen(reduced);
             }
         } else {
@@ -1529,7 +1585,7 @@ int ZBarcode_BarcodeName(int symbol_id, char name[32]) {
         { "BARCODE_PHARMA", BARCODE_PHARMA, 51 },
         { "BARCODE_PZN", BARCODE_PZN, 52 },
         { "BARCODE_PHARMA_TWO", BARCODE_PHARMA_TWO, 53 },
-        { "", -1, 54 },
+        { "BARCODE_CEPNET", BARCODE_CEPNET, 54 },
         { "BARCODE_PDF417", BARCODE_PDF417, 55 },
         { "BARCODE_PDF417COMP", BARCODE_PDF417COMP, 56 },
         { "BARCODE_MAXICODE", BARCODE_MAXICODE, 57 },
@@ -1621,6 +1677,7 @@ int ZBarcode_BarcodeName(int symbol_id, char name[32]) {
         { "BARCODE_UPNQR", BARCODE_UPNQR, 143 },
         { "BARCODE_ULTRA", BARCODE_ULTRA, 144 },
         { "BARCODE_RMQR", BARCODE_RMQR, 145 },
+        { "BARCODE_BC412", BARCODE_BC412, 146 },
     };
 
     name[0] = '\0';
@@ -1717,8 +1774,8 @@ unsigned int ZBarcode_Cap(int symbol_id, unsigned int cap_flag) {
         switch (symbol_id) {
             case BARCODE_QRCODE:
             case BARCODE_MICROQR:
-            //case BARCODE_HIBC_QR: Note character set restricted to ASCII subset
-            //case BARCODE_UPNQR: Note does not use Kanji mode
+            /* case BARCODE_HIBC_QR: Note character set restricted to ASCII subset */
+            /* case BARCODE_UPNQR: Note does not use Kanji mode */
             case BARCODE_RMQR:
             case BARCODE_HANXIN:
             case BARCODE_GRIDMATRIX:
