@@ -767,6 +767,8 @@ static void test_utf8_to_eci_ascii(void) {
             assert_zero(memcmp(data[i].data, dest, length), "i:%d memcmp != 0\n", i);
         }
     }
+
+    testFinish();
 }
 
 static void test_utf8_to_eci_utf16be(void) {
@@ -790,8 +792,11 @@ static void test_utf8_to_eci_utf16be(void) {
         /*  7*/ { "\355\277\277", -1, ZINT_ERROR_INVALID_DATA, -1, NULL }, // U+DFFF (edbfbf) surrogate half not allowed
         /*  8*/ { "\356\200\200", -1, 0, 2, "\340\000" }, // U+E000 (ee8080)
         /*  9*/ { "\360\220\200\200", -1, 0, 4, "\330\000\334\000" }, // U+10000 maps to surrogate pair
-        /* 10*/ { "\364\217\277\277", -1, 0, 4, "\333\377\337\377" }, // U+10FFFF maps to surrogate pair
-        /* 11*/ { "\364\220\200\200", -1, ZINT_ERROR_INVALID_DATA, -1, NULL }, // Non-Unicode 0x110000 not allowed
+        /* 10*/ { "\360\220\217\277", -1, 0, 4, "\330\000\337\377" }, // U+103FF maps to surrogate pair
+        /* 11*/ { "\364\200\200\200", -1, 0, 4, "\333\300\334\000" }, // U+100000 maps to surrogate pair
+        /* 12*/ { "\364\217\260\200", -1, 0, 4, "\333\377\334\000" }, // U+10FC00 maps to surrogate pair
+        /* 13*/ { "\364\217\277\277", -1, 0, 4, "\333\377\337\377" }, // U+10FFFF maps to surrogate pair
+        /* 14*/ { "\364\220\200\200", -1, ZINT_ERROR_INVALID_DATA, -1, NULL }, // Non-Unicode 0x110000 not allowed
     };
     int data_size = ARRAY_SIZE(data);
     int i, length, ret;
@@ -825,6 +830,8 @@ static void test_utf8_to_eci_utf16be(void) {
             }
         }
     }
+
+    testFinish();
 }
 
 static void test_utf8_to_eci_utf16le(void) {
@@ -848,8 +855,11 @@ static void test_utf8_to_eci_utf16le(void) {
         /*  7*/ { "\355\277\277", -1, ZINT_ERROR_INVALID_DATA, -1, NULL }, // U+DFFF (edbfbf) surrogate half not allowed
         /*  8*/ { "\356\200\200", -1, 0, 2, "\000\340" }, // U+E000 (ee8080)
         /*  9*/ { "\360\220\200\200", -1, 0, 4, "\000\330\000\334" }, // U+10000 maps to surrogate pair
-        /* 10*/ { "\364\217\277\277", -1, 0, 4, "\377\333\377\337" }, // U+10FFFF maps to surrogate pair
-        /* 11*/ { "\364\220\200\200", -1, ZINT_ERROR_INVALID_DATA, -1, NULL }, // Non-Unicode 0x110000 not allowed
+        /* 10*/ { "\360\220\217\277", -1, 0, 4, "\000\330\377\337" }, // U+103FF maps to surrogate pair
+        /* 11*/ { "\364\200\200\200", -1, 0, 4, "\300\333\000\334" }, // U+100000 maps to surrogate pair
+        /* 12*/ { "\364\217\260\200", -1, 0, 4, "\377\333\000\334" }, // U+10FC00 maps to surrogate pair
+        /* 13*/ { "\364\217\277\277", -1, 0, 4, "\377\333\377\337" }, // U+10FFFF maps to surrogate pair
+        /* 14*/ { "\364\220\200\200", -1, ZINT_ERROR_INVALID_DATA, -1, NULL }, // Non-Unicode 0x110000 not allowed
     };
     int data_size = ARRAY_SIZE(data);
     int i, length, ret;
@@ -883,6 +893,8 @@ static void test_utf8_to_eci_utf16le(void) {
             }
         }
     }
+
+    testFinish();
 }
 
 static void test_utf8_to_eci_utf32be(void) {
@@ -943,6 +955,8 @@ static void test_utf8_to_eci_utf32be(void) {
             }
         }
     }
+
+    testFinish();
 }
 
 static void test_utf8_to_eci_utf32le(void) {
@@ -1003,6 +1017,8 @@ static void test_utf8_to_eci_utf32le(void) {
             }
         }
     }
+
+    testFinish();
 }
 
 static void test_utf8_to_eci_sjis(void) {
@@ -1055,6 +1071,8 @@ static void test_utf8_to_eci_sjis(void) {
             assert_nonzero(out_length <= eci_length, "i:%d out_length %d > eci_length %d\n", i, out_length, eci_length);
         }
     }
+
+    testFinish();
 }
 
 static void test_utf8_to_eci_big5(void) {
@@ -1097,6 +1115,8 @@ static void test_utf8_to_eci_big5(void) {
             assert_nonzero(out_length <= eci_length, "i:%d out_length %d > eci_length %d\n", i, out_length, eci_length);
         }
     }
+
+    testFinish();
 }
 
 static void test_utf8_to_eci_gb2312(void) {
@@ -1139,6 +1159,8 @@ static void test_utf8_to_eci_gb2312(void) {
             assert_nonzero(out_length <= eci_length, "i:%d out_length %d > eci_length %d\n", i, out_length, eci_length);
         }
     }
+
+    testFinish();
 }
 
 static void test_utf8_to_eci_euc_kr(void) {
@@ -1181,6 +1203,8 @@ static void test_utf8_to_eci_euc_kr(void) {
             assert_nonzero(out_length <= eci_length, "i:%d out_length %d > eci_length %d\n", i, out_length, eci_length);
         }
     }
+
+    testFinish();
 }
 
 static void test_utf8_to_eci_gbk(void) {
@@ -1223,6 +1247,8 @@ static void test_utf8_to_eci_gbk(void) {
             assert_nonzero(out_length <= eci_length, "i:%d out_length %d > eci_length %d\n", i, out_length, eci_length);
         }
     }
+
+    testFinish();
 }
 
 static void test_utf8_to_eci_gb18030(void) {
@@ -1265,6 +1291,8 @@ static void test_utf8_to_eci_gb18030(void) {
             assert_nonzero(out_length <= eci_length, "i:%d out_length %d > eci_length %d\n", i, out_length, eci_length);
         }
     }
+
+    testFinish();
 }
 
 static void test_is_eci_convertible_segs(int index) {
