@@ -1,6 +1,6 @@
 /*
     libzint - the open source barcode library
-    Copyright (C) 2020 - 2021 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2020-2022 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -27,7 +27,7 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
-/* vim: set ts=4 sw=4 et : */
+/* SPDX-License-Identifier: BSD-3-Clause */
 
 #include "testcommon.h"
 #include <errno.h>
@@ -44,7 +44,7 @@ static void test_pixel_plot(int index, int debug) {
         int repeat;
         int ret;
     };
-    // s/\/\*[ 0-9]*\*\//\=printf("\/*%3d*\/", line(".") - line("'<"))
+    /* s/\/\*[ 0-9]*\*\//\=printf("\/\*%3d*\/", line(".") - line("'<")): */
     struct item data[] = {
         /*  0*/ { 1, 1, "1", 0, 0 },
         /*  1*/ { 2, 1, "11", 0, 0 },
@@ -337,7 +337,7 @@ struct wpng_error_type {
     jmp_buf jmpbuf;
 };
 
-STATIC_UNLESS_ZINT_TEST void wpng_error_handler(png_structp png_ptr, png_const_charp msg);
+INTERNAL void wpng_error_handler_test(png_structp png_ptr, png_const_charp msg);
 
 static void test_wpng_error_handler(void) {
     int ret;
@@ -352,18 +352,18 @@ static void test_wpng_error_handler(void) {
 
     wpng_error.symbol = &symbol;
 
-    // Create empty file
-    (void) remove(filename); // In case junk hanging around
+    /* Create empty file */
+    (void) remove(filename); /* In case junk hanging around */
     fp = fopen(filename, "w+");
     assert_nonnull(fp, "fopen(%s) failed\n", filename);
     ret = fclose(fp);
     assert_zero(ret, "fclose(%s) %d != 0\n", filename, ret);
 
-    // Re-open for read, which will cause libpng to error
+    /* Re-open for read, which will cause libpng to error */
     fp = fopen(filename, "r");
     assert_nonnull(fp, "fopen(%s) for read failed\n", filename);
 
-    png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, &wpng_error, wpng_error_handler, NULL);
+    png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, &wpng_error, wpng_error_handler_test, NULL);
     assert_nonnull(png_ptr, "png_create_write_struct failed\n");
 
     info_ptr = png_create_info_struct(png_ptr);
@@ -378,7 +378,7 @@ static void test_wpng_error_handler(void) {
     } else {
         png_init_io(png_ptr, fp);
 
-        // This should fail and jmp to setjmp
+        /* This should fail and jmp to setjmp */
         png_write_info(png_ptr, info_ptr);
         assert_zero(1, "libpng error setjmp failed\n");
     }
@@ -386,7 +386,7 @@ static void test_wpng_error_handler(void) {
     testFinish();
 }
 
-// Check compliant height printable for max CODABLOCKF with 44 rows * ((62 cols) * 0.55 + 3)) = 1632.4
+/* Check compliant height printable for max CODABLOCKF with 44 rows * ((62 cols) * 0.55 + 3)) = 1632.4 */
 static void test_large_compliant_height(void) {
     int ret;
     struct zint_symbol *symbol = NULL;
@@ -429,3 +429,5 @@ int main(int argc, char *argv[]) {
 
     return 0;
 }
+
+/* vim: set ts=4 sw=4 et : */
