@@ -1,6 +1,6 @@
 /*
     libzint - the open source barcode library
-    Copyright (C) 2020 - 2021 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2020-2022 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -27,7 +27,6 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
-/* vim: set ts=4 sw=4 et : */
 
 #include "testcommon.h"
 
@@ -267,7 +266,7 @@ static void test_encode(int index, int generate, int debug) {
                 assert_zero(ret, "i:%d testUtilModulesCmp ret %d != 0 width %d row %d (%s)\n", i, ret, width, row, data[i].data);
 
                 if (do_bwipp && testUtilCanBwipp(i, symbol, -1, -1, -1, debug)) {
-                    ret = testUtilBwipp(i, symbol, -1, -1, -1, data[i].data, length, NULL, bwipp_buf, sizeof(bwipp_buf));
+                    ret = testUtilBwipp(i, symbol, -1, -1, -1, data[i].data, length, NULL, bwipp_buf, sizeof(bwipp_buf), NULL);
                     assert_zero(ret, "i:%d %s testUtilBwipp ret %d != 0\n", i, testUtilBarcodeName(symbol->symbology), ret);
 
                     ret = testUtilBwippCmp(symbol, bwipp_msg, bwipp_buf, data[i].expected);
@@ -330,6 +329,62 @@ static void test_fuzz(int index, int debug) {
     testFinish();
 }
 
+static const char TeleTable[128][16] = {
+    { "31313131"       }, { "1131313111"     }, { "33313111"     }, { "1111313131"     },
+    { "3111313111"     }, { "11333131"       }, { "13133131"     }, { "111111313111"   },
+    { "31333111"       }, { "1131113131"     }, { "33113131"     }, { "1111333111"     },
+    { "3111113131"     }, { "1113133111"     }, { "1311133111"   }, { "111111113131"   },
+    { "3131113111"     }, { "11313331"       }, { "333331"       }, { "111131113111"   },
+    { "31113331"       }, { "1133113111"     }, { "1313113111"   }, { "1111113331"     },
+    { "31131331"       }, { "113111113111"   }, { "3311113111"   }, { "1111131331"     },
+    { "311111113111"   }, { "1113111331"     }, { "1311111331"   }, { "11111111113111" },
+    { "31313311"       }, { "1131311131"     }, { "33311131"     }, { "1111313311"     },
+    { "3111311131"     }, { "11333311"       }, { "13133311"     }, { "111111311131"   },
+    { "31331131"       }, { "1131113311"     }, { "33113311"     }, { "1111331131"     },
+    { "3111113311"     }, { "1113131131"     }, { "1311131131"   }, { "111111113311"   },
+    { "3131111131"     }, { "1131131311"     }, { "33131311"     }, { "111131111131"   },
+    { "3111131311"     }, { "1133111131"     }, { "1313111131"   }, { "111111131311"   },
+    { "3113111311"     }, { "113111111131"   }, { "3311111131"   }, { "111113111311"   },
+    { "311111111131"   }, { "111311111311"   }, { "131111111311" }, { "11111111111131" },
+    { "3131311111"     }, { "11313133"       }, { "333133"       }, { "111131311111"   },
+    { "31113133"       }, { "1133311111"     }, { "1313311111"   }, { "1111113133"     },
+    { "313333"         }, { "113111311111"   }, { "3311311111"   }, { "11113333"       },
+    { "311111311111"   }, { "11131333"       }, { "13111333"     }, { "11111111311111" },
+    { "31311133"       }, { "1131331111"     }, { "33331111"     }, { "1111311133"     },
+    { "3111331111"     }, { "11331133"       }, { "13131133"     }, { "111111331111"   },
+    { "3113131111"     }, { "1131111133"     }, { "33111133"     }, { "111113131111"   },
+    { "3111111133"     }, { "111311131111"   }, { "131111131111" }, { "111111111133"   },
+    { "31311313"       }, { "113131111111"   }, { "3331111111"   }, { "1111311313"     },
+    { "311131111111"   }, { "11331313"       }, { "13131313"     }, { "11111131111111" },
+    { "3133111111"     }, { "1131111313"     }, { "33111313"     }, { "111133111111"   },
+    { "3111111313"     }, { "111313111111"   }, { "131113111111" }, { "111111111313"   },
+    { "313111111111"   }, { "1131131113"     }, { "33131113"     }, { "11113111111111" },
+    { "3111131113"     }, { "113311111111"   }, { "131311111111" }, { "111111131113"   },
+    { "3113111113"     }, { "11311111111111" }, { "331111111111" }, { "111113111113"   },
+    { "31111111111111" }, { "111311111113"   }, { "131111111113" },
+    {'1','1','1','1','1','1','1','1','1','1','1','1','1','1','1','1'},
+};
+
+// Dummy to generate lengths table
+static void test_generate_lens(int generate) {
+
+    int i;
+
+    if (!generate) {
+        return;
+    }
+
+    printf("static const char TeleLens[128] = {");
+    for (i = 0; i < 127; i++) {
+        if ((i % 16) == 0) {
+            printf("\n    %2d,", (int) strlen(TeleTable[i]));
+        } else {
+            printf(" %2d,", (int) strlen(TeleTable[i]));
+        }
+    }
+    printf(" 16\n};\n");
+}
+
 int main(int argc, char *argv[]) {
 
     testFunction funcs[] = { /* name, func, has_index, has_generate, has_debug */
@@ -338,6 +393,7 @@ int main(int argc, char *argv[]) {
         { "test_input", test_input, 1, 0, 1 },
         { "test_encode", test_encode, 1, 1, 1 },
         { "test_fuzz", test_fuzz, 1, 0, 1 },
+        { "test_generate_lens", test_generate_lens, 0, 1, 0 },
     };
 
     testRun(argc, argv, funcs, ARRAY_SIZE(funcs));
@@ -346,3 +402,5 @@ int main(int argc, char *argv[]) {
 
     return 0;
 }
+
+/* vim: set ts=4 sw=4 et : */
