@@ -1,7 +1,7 @@
-/*  zint.h - definitions for libzint
-
+/*  zint.h - definitions for libzint */
+/*
     libzint - the open source barcode library
-    Copyright (C) 2009-2021 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2009-2022 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -28,7 +28,8 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
-/* vim: set ts=4 sw=4 et : */
+/* SPDX-License-Identifier: BSD-3-Clause */
+
 /*
  * For version, see "zintconfig.h"
  * For documentation, see "../docs/manual.txt"
@@ -52,9 +53,10 @@ extern "C" {
         int rotation;       /* 0, 90, 180, 270 degrees */
         struct zint_vector_hexagon *next; /* Pointer to next hexagon */
     };
-    
+
     struct zint_vector_string {
-        float x, y, fsize;  /* x, y position relative to halign; font size */
+        float x, y;         /* x, y position relative to halign */
+        float fsize;        /* Font size */
         float width;        /* Suggested string width, may be 0 if none recommended */
         int length;         /* Number of characters */
         int rotation;       /* 0, 90, 180, 270 degrees */
@@ -64,8 +66,10 @@ extern "C" {
     };
 
     struct zint_vector_circle {
-        float x, y, diameter;
-        int colour;         /* Non-zero for draw with background colour */
+        float x, y;
+        float diameter;     /* Circle diameter. Does not include width (if any) */
+        float width;        /* Width of circle perimeter (circumference). 0 for fill (disc) */
+        int colour;         /* Non-zero for draw with background colour (else draw with foreground colour) */
         struct zint_vector_circle *next; /* Pointer to next circle */
     };
 
@@ -78,10 +82,18 @@ extern "C" {
         struct zint_vector_circle *circles; /* Pointer to first circle */
     };
 
+    /* Structured Append info - ignored unless `zint_structapp.count` is set to non-zero value */
+    struct zint_structapp {
+        int index;          /* Position in Structured Append sequence, 1-based. Must be <= `count` */
+        int count;          /* Number of symbols in Structured Append sequence. Set >= 2 to add SA Info */
+        char id[32];        /* Optional ID to distinguish sequence, ASCII, NUL-terminated unless max 32 long */
+    };
+
     /* Main symbol structure */
     struct zint_symbol {
         int symbology;      /* Symbol to use (see BARCODE_XXX below) */
-        float height;       /* Height in X-dimensions (ignored for fixed-width barcodes) */
+        float height;       /* Barcode height in X-dimensions (ignored for fixed-width barcodes) */
+        float scale;        /* Scale factor when printing barcode, i.e. adjusts X-dimension. Default 1 */
         int whitespace_width; /* Width in X-dimensions of whitespace to left & right of barcode */
         int whitespace_height; /* Height in X-dimensions of whitespace above & below the barcode */
         int border_width;   /* Size of border in X-dimensions */
@@ -91,7 +103,7 @@ extern "C" {
         char *fgcolor;      /* Pointer to fgcolour (alternate spelling) */
         char *bgcolor;      /* Pointer to bgcolour (alternate spelling) */
         char outfile[256];  /* Name of file to output to, NUL-terminated. Default "out.png" ("out.gif" if NO_PNG) */
-        float scale;        /* Scale factor when printing barcode */
+        char primary[128];  /* Primary message data (MaxiCode, Composite), NUL-terminated */
         int option_1;       /* Symbol-specific options (see "../docs/manual.txt") */
         int option_2;       /* Symbol-specific options */
         int option_3;       /* Symbol-specific options */
@@ -99,11 +111,15 @@ extern "C" {
         int fontsize;       /* Unused */
         int input_mode;     /* Encoding of input data (see DATA_MODE etc below). Default DATA_MODE */
         int eci;            /* Extended Channel Interpretation. Default 0 (none) */
+        float dot_size;     /* Size of dots used in BARCODE_DOTTY_MODE. Default 0.8 */
+        float guard_descent; /* Height in X-dimensions that EAN/UPC guard bars descend. Default 5 */
+        struct zint_structapp structapp; /* Structured Append info. Default structapp.count 0 (none) */
+        int warn_level;     /* Affects error/warning value returned by Zint API (see WARN_XXX below) */
+        int debug;          /* Debugging flags */
         unsigned char text[128]; /* Human Readable Text (if any), UTF-8, NUL-terminated (output only) */
         int rows;           /* Number of rows used by the symbol (output only) */
         int width;          /* Width of the generated symbol (output only) */
-        char primary[128];  /* Primary message data (MaxiCode, Composite), NUL-terminated */
-        unsigned char encoded_data[200][143]; /* Encoded data (output only). Allows for rows of 1144 modules */
+        unsigned char encoded_data[200][144]; /* Encoded data (output only). Allows for rows of 1152 modules */
         float row_height[200]; /* Heights of rows (output only). Allows for 200 row DotCode */
         char errtxt[100];   /* Error message if an error or warning occurs, NUL-terminated (output only) */
         unsigned char *bitmap; /* Stored bitmap image (raster output only) */
@@ -111,10 +127,14 @@ extern "C" {
         int bitmap_height;  /* Height of bitmap image (raster output only) */
         unsigned char *alphamap; /* Array of alpha values used (raster output only) */
         unsigned int bitmap_byte_length; /* Size of BMP bitmap data (raster output only) */
-        float dot_size;     /* Size of dots used in BARCODE_DOTTY_MODE */
         struct zint_vector *vector; /* Pointer to vector header (vector output only) */
-        int debug;          /* Debugging flags */
-        int warn_level;     /* Affects error/warning value returned by Zint API (see WARN_XXX below) */
+    };
+
+    /* Segment for use with `ZBarcode_Encode_Segs()` below */
+    struct zint_seg {
+        unsigned char *source; /* Data to encode */
+        int length;         /* Length of `source`. If 0, `source` must be NUL-terminated */
+        int eci;            /* Extended Channel Interpretation */
     };
 
 /* Symbologies (`symbol->symbology`) */
@@ -151,13 +171,14 @@ extern "C" {
 #define BARCODE_UPCA_CHK        35  /* UPC-A + Check Digit */
 #define BARCODE_UPCE            37  /* UPC-E */
 #define BARCODE_UPCE_CHK        38  /* UPC-E + Check Digit */
-#define BARCODE_POSTNET         40  /* USPS POSTNET */
+#define BARCODE_POSTNET         40  /* USPS (U.S. Postal Service) POSTNET */
 #define BARCODE_MSI_PLESSEY     47  /* MSI Plessey */
 #define BARCODE_FIM             49  /* Facing Identification Mark */
 #define BARCODE_LOGMARS         50  /* LOGMARS */
 #define BARCODE_PHARMA          51  /* Pharmacode One-Track */
 #define BARCODE_PZN             52  /* Pharmazentralnummer */
 #define BARCODE_PHARMA_TWO      53  /* Pharmacode Two-Track */
+#define BARCODE_CEPNET          54  /* Brazilian CEPNet Postal Code */
 #define BARCODE_PDF417          55  /* PDF417 */
 #define BARCODE_PDF417COMP      56  /* Compact PDF417 (Truncated PDF417) */
 #define BARCODE_PDF417TRUNC     56  /* Legacy */
@@ -169,7 +190,7 @@ extern "C" {
 #define BARCODE_AUSROUTE        67  /* Australia Post Routing */
 #define BARCODE_AUSREDIRECT     68  /* Australia Post Redirection */
 #define BARCODE_ISBNX           69  /* ISBN */
-#define BARCODE_RM4SCC          70  /* Royal Mail 4 State */
+#define BARCODE_RM4SCC          70  /* Royal Mail 4-State Customer Code */
 #define BARCODE_DATAMATRIX      71  /* Data Matrix (ECC200) */
 #define BARCODE_EAN14           72  /* EAN-14 */
 #define BARCODE_VIN             73  /* Vehicle Identification Number */
@@ -187,7 +208,7 @@ extern "C" {
 #define BARCODE_MICROPDF417     84  /* MicroPDF417 */
 #define BARCODE_USPS_IMAIL      85  /* USPS Intelligent Mail (OneCode) */
 #define BARCODE_ONECODE         85  /* Legacy */
-#define BARCODE_PLESSEY         86  /* Plessey Code */
+#define BARCODE_PLESSEY         86  /* UK Plessey */
 
     /* Tbarcode 8 codes */
 #define BARCODE_TELEPEN_NUM     87  /* Telepen Numeric */
@@ -199,7 +220,7 @@ extern "C" {
 #define BARCODE_MICROQR         97  /* Micro QR Code */
 
     /* Tbarcode 9 codes */
-#define BARCODE_HIBC_128        98  /* HIBC Code 128 */
+#define BARCODE_HIBC_128        98  /* HIBC (Health Industry Barcode) Code 128 */
 #define BARCODE_HIBC_39         99  /* HIBC Code 39 */
 #define BARCODE_HIBC_DM         102 /* HIBC Data Matrix */
 #define BARCODE_HIBC_QR         104 /* HIBC QR Code */
@@ -213,7 +234,7 @@ extern "C" {
 #define BARCODE_HANXIN          116 /* Han Xin (Chinese Sensible) Code */
 
     /* Tbarcode 11 codes */
-#define BARCODE_MAILMARK        121 /* Royal Mail 4-state Mailmark */
+#define BARCODE_MAILMARK        121 /* Royal Mail 4-State Mailmark */
 
     /* Zint specific */
 #define BARCODE_AZRUNE          128 /* Aztec Runes */
@@ -241,78 +262,93 @@ extern "C" {
 #define BARCODE_UPNQR           143 /* UPNQR (Univerzalnega Plačilnega Naloga QR) */
 #define BARCODE_ULTRA           144 /* Ultracode */
 #define BARCODE_RMQR            145 /* Rectangular Micro QR Code (rMQR) */
+#define BARCODE_BC412           146 /* IBM BC412 (SEMI T1-95) */
+#define BARCODE_LAST            146 /* Max barcode number marker, not barcode */
 
 /* Output options (`symbol->output_options`) */
-#define BARCODE_NO_ASCII        1   /* Legacy (no-op) */
-#define BARCODE_BIND            2   /* Boundary bars above & below the symbol and between stacked symbols */
-#define BARCODE_BOX             4   /* Box around symbol */
-#define BARCODE_STDOUT          8   /* Output to stdout */
-#define READER_INIT             16  /* Reader Initialisation (Programming) */
-#define SMALL_TEXT              32  /* Use smaller font */
-#define BOLD_TEXT               64  /* Use bold font */
-#define CMYK_COLOUR             128 /* CMYK colour space (Encapsulated PostScript and TIF) */
-#define BARCODE_DOTTY_MODE      256 /* Plot a matrix symbol using dots rather than squares */
-#define GS1_GS_SEPARATOR        512 /* Use GS instead of FNC1 as GS1 separator (Data Matrix) */
-#define OUT_BUFFER_INTERMEDIATE 1024 /* Return ASCII values in bitmap buffer (OUT_BUFFER only) */
+#define BARCODE_NO_ASCII        0x0001  /* Legacy (no-op) */
+#define BARCODE_BIND            0x0002  /* Boundary bars above & below the symbol and between stacked symbols */
+#define BARCODE_BOX             0x0004  /* Box around symbol */
+#define BARCODE_STDOUT          0x0008  /* Output to stdout */
+#define READER_INIT             0x0010  /* Reader Initialisation (Programming) */
+#define SMALL_TEXT              0x0020  /* Use smaller font */
+#define BOLD_TEXT               0x0040  /* Use bold font */
+#define CMYK_COLOUR             0x0080  /* CMYK colour space (Encapsulated PostScript and TIF) */
+#define BARCODE_DOTTY_MODE      0x0100  /* Plot a matrix symbol using dots rather than squares */
+#define GS1_GS_SEPARATOR        0x0200  /* Use GS instead of FNC1 as GS1 separator (Data Matrix) */
+#define OUT_BUFFER_INTERMEDIATE 0x0400  /* Return ASCII values in bitmap buffer (OUT_BUFFER only) */
+#define BARCODE_QUIET_ZONES     0x0800  /* Add compliant quiet zones (additional to any specified whitespace) */
+                                        /* Note: CODE16K, CODE49, CODABLOCKF, ITF14, EAN/UPC have default quiet zones
+                                         */
+#define BARCODE_NO_QUIET_ZONES  0x1000  /* Disable quiet zones, notably those with defaults as listed above */
+#define COMPLIANT_HEIGHT        0x2000  /* Warn if height not compliant and use standard height (if any) as default */
 
 /* Input data types (`symbol->input_mode`) */
-#define DATA_MODE               0   /* Binary */
-#define UNICODE_MODE            1   /* UTF-8 */
-#define GS1_MODE                2   /* GS1 */
+#define DATA_MODE               0       /* Binary */
+#define UNICODE_MODE            1       /* UTF-8 */
+#define GS1_MODE                2       /* GS1 */
 /* The following may be OR-ed with above */
-#define ESCAPE_MODE             8   /* Process escape sequences */
-#define GS1PARENS_MODE          16  /* Process parentheses as GS1 AI delimiters (instead of square brackets) */
-#define GS1NOCHECK_MODE         32  /* Do not check validity of GS1 data (except that printable ASCII only) */
+#define ESCAPE_MODE             0x0008  /* Process escape sequences */
+#define GS1PARENS_MODE          0x0010  /* Process parentheses as GS1 AI delimiters (instead of square brackets) */
+#define GS1NOCHECK_MODE         0x0020  /* Do not check validity of GS1 data (except that printable ASCII only) */
+#define HEIGHTPERROW_MODE       0x0040  /* Interpret `height` as per-row rather than as overall height */
+#define FAST_MODE               0x0080  /* Use faster if less optimal encodation for symbologies that support it */
+                                        /* Note: only DATAMATRIX currently */
 
 /* Data Matrix specific options (`symbol->option_3`) */
-#define DM_SQUARE               100 /* Only consider square versions on automatic symbol size selection */
-#define DM_DMRE                 101 /* Consider DMRE versions on automatic symbol size selection */
+#define DM_SQUARE               100     /* Only consider square versions on automatic symbol size selection */
+#define DM_DMRE                 101     /* Consider DMRE versions on automatic symbol size selection */
 
 /* QR, Han Xin, Grid Matrix specific options (`symbol->option_3`) */
-#define ZINT_FULL_MULTIBYTE     200 /* Enable Kanji/Hanzi compression for Latin-1 & binary data */
+#define ZINT_FULL_MULTIBYTE     200     /* Enable Kanji/Hanzi compression for Latin-1 & binary data */
 
 /* Ultracode specific option (`symbol->option_3`) */
-#define ULTRA_COMPRESSION       128 /* Enable Ultracode compression (experimental) */
+#define ULTRA_COMPRESSION       128     /* Enable Ultracode compression (experimental) */
 
 /* Warning and error conditions (API return values) */
-#define ZINT_WARN_INVALID_OPTION        2   /* Invalid option given but overridden by Zint */
-#define ZINT_WARN_USES_ECI              3   /* Automatic ECI inserted by Zint */
-#define ZINT_WARN_NONCOMPLIANT          4   /* Symbol created not compliant with standards */
-#define ZINT_ERROR                      5   /* Warn/error marker, not returned */
-#define ZINT_ERROR_TOO_LONG             5   /* Input data wrong length */
-#define ZINT_ERROR_INVALID_DATA         6   /* Input data incorrect */
-#define ZINT_ERROR_INVALID_CHECK        7   /* Input check digit incorrect */
-#define ZINT_ERROR_INVALID_OPTION       8   /* Incorrect option given */
-#define ZINT_ERROR_ENCODING_PROBLEM     9   /* Internal error (should not happen) */
-#define ZINT_ERROR_FILE_ACCESS          10  /* Error opening output file */
-#define ZINT_ERROR_MEMORY               11  /* Memory allocation (malloc) failure */
-#define ZINT_ERROR_FILE_WRITE           12  /* Error writing to output file */
-#define ZINT_ERROR_USES_ECI             13  /* Error counterpart of warning if WARN_FAIL_ALL set (see below) */
-#define ZINT_ERROR_NONCOMPLIANT         14  /* Error counterpart of warning if WARN_FAIL_ALL set */
+#define ZINT_WARN_INVALID_OPTION    2   /* Invalid option given but overridden by Zint */
+#define ZINT_WARN_USES_ECI          3   /* Automatic ECI inserted by Zint */
+#define ZINT_WARN_NONCOMPLIANT      4   /* Symbol created not compliant with standards */
+#define ZINT_ERROR                  5   /* Warn/error marker, not returned */
+#define ZINT_ERROR_TOO_LONG         5   /* Input data wrong length */
+#define ZINT_ERROR_INVALID_DATA     6   /* Input data incorrect */
+#define ZINT_ERROR_INVALID_CHECK    7   /* Input check digit incorrect */
+#define ZINT_ERROR_INVALID_OPTION   8   /* Incorrect option given */
+#define ZINT_ERROR_ENCODING_PROBLEM 9   /* Internal error (should not happen) */
+#define ZINT_ERROR_FILE_ACCESS      10  /* Error opening output file */
+#define ZINT_ERROR_MEMORY           11  /* Memory allocation (malloc) failure */
+#define ZINT_ERROR_FILE_WRITE       12  /* Error writing to output file */
+#define ZINT_ERROR_USES_ECI         13  /* Error counterpart of warning if WARN_FAIL_ALL set (see below) */
+#define ZINT_ERROR_NONCOMPLIANT     14  /* Error counterpart of warning if WARN_FAIL_ALL set */
 
 /* Warning warn (`symbol->warn_level`) */
 #define WARN_DEFAULT            0  /* Default behaviour */
 #define WARN_FAIL_ALL           2  /* Treat warning as error */
 
 /* Capability flags (ZBarcode_Cap() `cap_flag`) */
-#define ZINT_CAP_HRT            0x0001  /* Prints Human Readable Text? */
-#define ZINT_CAP_STACKABLE      0x0002  /* Is stackable? */
-#define ZINT_CAP_EXTENDABLE     0x0004  /* Is extendable with add-on data? (Is UPC/EAN?) */
-#define ZINT_CAP_COMPOSITE      0x0008  /* Can have composite data? */
-#define ZINT_CAP_ECI            0x0010  /* Supports Extended Channel Interpretations? */
-#define ZINT_CAP_GS1            0x0020  /* Supports GS1 data? */
-#define ZINT_CAP_DOTTY          0x0040  /* Can be output as dots? */
-#define ZINT_CAP_FIXED_RATIO    0x0100  /* Has fixed width-to-height (aspect) ratio? */
-#define ZINT_CAP_READER_INIT    0x0200  /* Supports Reader Initialisation? */
-#define ZINT_CAP_FULL_MULTIBYTE 0x0400  /* Supports full-multibyte option? */
-#define ZINT_CAP_MASK           0x0800  /* Is mask selectable? */
+#define ZINT_CAP_HRT                0x0001  /* Prints Human Readable Text? */
+#define ZINT_CAP_STACKABLE          0x0002  /* Is stackable? */
+#define ZINT_CAP_EXTENDABLE         0x0004  /* Is extendable with add-on data? (Is EAN/UPC?) */
+#define ZINT_CAP_COMPOSITE          0x0008  /* Can have composite data? */
+#define ZINT_CAP_ECI                0x0010  /* Supports Extended Channel Interpretations? */
+#define ZINT_CAP_GS1                0x0020  /* Supports GS1 data? */
+#define ZINT_CAP_DOTTY              0x0040  /* Can be output as dots? */
+#define ZINT_CAP_QUIET_ZONES        0x0080  /* Has default quiet zones? */
+#define ZINT_CAP_FIXED_RATIO        0x0100  /* Has fixed width-to-height (aspect) ratio? */
+#define ZINT_CAP_READER_INIT        0x0200  /* Supports Reader Initialisation? */
+#define ZINT_CAP_FULL_MULTIBYTE     0x0400  /* Supports full-multibyte option? */
+#define ZINT_CAP_MASK               0x0800  /* Is mask selectable? */
+#define ZINT_CAP_STRUCTAPP          0x1000  /* Supports Structured Append? */
+#define ZINT_CAP_COMPLIANT_HEIGHT   0x2000  /* Has compliant height? */
 
 /* The largest amount of data that can be encoded is 4350 4-byte UTF-8 chars in Han Xin Code */
 #define ZINT_MAX_DATA_LEN       17400
+/* Maximum number of segments allowed for (`seg_count`) */
+#define ZINT_MAX_SEG_COUNT      256
 
-/* Debug flags (debug) */
-#define ZINT_DEBUG_PRINT        1   /* Print debug info (if any) to stdout */
-#define ZINT_DEBUG_TEST         2   /* For internal test use only */
+/* Debug flags (`symbol->debug`) */
+#define ZINT_DEBUG_PRINT        0x0001  /* Print debug info (if any) to stdout */
+#define ZINT_DEBUG_TEST         0x0002  /* For internal test use only */
 
 #ifdef _WIN32
 #  if defined(DLL_EXPORT) || defined(PIC) || defined(_USRDLL)
@@ -336,9 +372,13 @@ extern "C" {
     ZINT_EXTERN void ZBarcode_Delete(struct zint_symbol *symbol);
 
 
-    /* Encode a barcode. If `length` is 0, `source` must be NUL-terminated. */
+    /* Encode a barcode. If `length` is 0, `source` must be NUL-terminated */
     ZINT_EXTERN int ZBarcode_Encode(struct zint_symbol *symbol, const unsigned char *source, int length);
 
+    /* Encode a barcode with multiple ECI segments */
+    ZINT_EXTERN int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[],
+                        const int seg_count);
+
     /* Encode a barcode using input data from file `filename` */
     ZINT_EXTERN int ZBarcode_Encode_File(struct zint_symbol *symbol, const char *filename);
 
@@ -350,6 +390,10 @@ extern "C" {
     ZINT_EXTERN int ZBarcode_Encode_and_Print(struct zint_symbol *symbol, const unsigned char *source, int length,
                         int rotate_angle);
 
+    /* Encode a symbol with multiple ECI segments and output to file `symbol->outfile` */
+    ZINT_EXTERN int ZBarcode_Encode_Segs_and_Print(struct zint_symbol *symbol, const struct zint_seg segs[],
+                        const int seg_count, int rotate_angle);
+
     /* Encode a symbol using input data from file `filename` and output to file `symbol->outfile` */
     ZINT_EXTERN int ZBarcode_Encode_File_and_Print(struct zint_symbol *symbol, const char *filename,
                         int rotate_angle);
@@ -362,6 +406,10 @@ extern "C" {
     ZINT_EXTERN int ZBarcode_Encode_and_Buffer(struct zint_symbol *symbol, const unsigned char *source, int length,
                         int rotate_angle);
 
+    /* Encode a symbol with multiple ECI segments and output to memory as raster (`symbol->bitmap`) */
+    ZINT_EXTERN int ZBarcode_Encode_Segs_and_Buffer(struct zint_symbol *symbol, const struct zint_seg segs[],
+                        const int seg_count, int rotate_angle);
+
     /* Encode a symbol using input data from file `filename` and output to memory as raster (`symbol->bitmap`) */
     ZINT_EXTERN int ZBarcode_Encode_File_and_Buffer(struct zint_symbol *symbol, const char *filename,
                         int rotate_angle);
@@ -374,6 +422,10 @@ extern "C" {
     ZINT_EXTERN int ZBarcode_Encode_and_Buffer_Vector(struct zint_symbol *symbol, const unsigned char *source,
                         int length, int rotate_angle);
 
+    /* Encode a symbol with multiple ECI segments and output to memory as vector (`symbol->vector`) */
+    ZINT_EXTERN int ZBarcode_Encode_Segs_and_Buffer_Vector(struct zint_symbol *symbol, const struct zint_seg segs[],
+                        const int seg_count, int rotate_angle);
+
     /* Encode a symbol using input data from file `filename` and output to memory as vector (`symbol->vector`) */
     ZINT_EXTERN int ZBarcode_Encode_File_and_Buffer_Vector(struct zint_symbol *symbol, const char *filename,
                         int rotate_angle);
@@ -382,14 +434,19 @@ extern "C" {
     /* Is `symbol_id` a recognized symbology? */
     ZINT_EXTERN int ZBarcode_ValidID(int symbol_id);
 
+    /* Copy BARCODE_XXX name of `symbol_id` into `name` buffer, NUL-terminated.
+       Returns 0 if valid, non-zero (1 or -1) if not valid */
+    ZINT_EXTERN int ZBarcode_BarcodeName(int symbol_id, char name[32]);
+
     /* Return the capability flags for symbology `symbol_id` that match `cap_flag` */
     ZINT_EXTERN unsigned int ZBarcode_Cap(int symbol_id, unsigned int cap_flag);
 
     /* Return the version of Zint linked to */
-    ZINT_EXTERN int ZBarcode_Version();
+    ZINT_EXTERN int ZBarcode_Version(void);
 
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
+/* vim: set ts=4 sw=4 et : */
 #endif /* ZINT_H */
