@@ -1,7 +1,7 @@
 /* zint_tcl.c TCL binding for zint */
 /*
     zint - the open source tcl binding to the zint barcode library
-    Copyright (C) 2014 Harald Oehlmann <oehhar@users.sourceforge.net>
+    Copyright (C) 2014-2022 Harald Oehlmann <oehhar@users.sourceforge.net>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -28,7 +28,6 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
 */
-/* vim: set ts=4 sw=4 et : */
 /*
  History
 
@@ -122,6 +121,36 @@
 - Removed -wzpl, added -gs1nocheck
 - Made -format position independent
 - Tabs -> spaces
+2021-09-21 GL
+- Added -guarddescent option
+- iHeight check int -> double
+2021-09-24 GL
+- Added -quietzones and -noquietzones options
+2021-09-27 GL
+- Added -structapp
+- Split up -to parsing (could seg fault if given non-int for X0 or Y0)
+2021-10-05 GL
+- Added -compliantheight option
+2021-10-30 GL
+- Added PDF417 -rows
+2021-11-19 GL
+- Added -heightperrow option
+- Added DBAR_EXPSTK, CODE16K, CODE49 -rows
+2021-12-17 GL
+- Added -fast option
+2022-04-08 GL
+- Updated ECIs to AIM ITS/04-023:2022
+  Note changed names "unicode" -> "utf-16be", "euc-cn" -> "gb2312"
+2022-04-24 GL
+- Added -segN options
+- Added "invariant" and "binary" ECIs
+- Tcl_GetIndexFromObj() flags arg -> 0
+2022-05-12 GL
+- -vers maximum changed to 999 (DAFT)
+2022-07-03 GL
+- Added BC412
+2022-08-20 GL
+- Added CEPNet
 */
 
 #if defined(__WIN32__) || defined(_WIN32) || defined(WIN32)
@@ -161,7 +190,7 @@
 
 
 /*----------------------------------------------------------------------------*/
-/* >>>>> Hepler defines */
+/* >>>>> Helper defines */
 
 /* Two macros are necessary to not include the define name, but the value */
 #define STRING(x) #x
@@ -189,7 +218,7 @@ static int Encode(Tcl_Interp *interp, int objc,
 
 /* >> List of Codes */
 
-static char *s_code_list[] = {
+static const char *s_code_list[] = {
     "Code11",
     "Standard2of5",
     "Interleaved2of5",
@@ -224,6 +253,7 @@ static char *s_code_list[] = {
     "Pharma",
     "PZN",
     "PharmaTwo",
+    "CEPNet",
     "PDF417",
     "PDF417Compact",
     "MaxiCode",
@@ -285,9 +315,10 @@ static char *s_code_list[] = {
     "UPNQR",
     "UltraCode",
     "rMQR",
+    "BC412",
     NULL};
 
-static int s_code_number[] = {
+static const int s_code_number[] = {
     BARCODE_CODE11,
     BARCODE_C25STANDARD,
     BARCODE_C25INTER,
@@ -322,6 +353,7 @@ static int s_code_number[] = {
     BARCODE_PHARMA,
     BARCODE_PZN,
     BARCODE_PHARMA_TWO,
+    BARCODE_CEPNET,
     BARCODE_PDF417,
     BARCODE_PDF417COMP,
     BARCODE_MAXICODE,
@@ -383,13 +415,14 @@ static int s_code_number[] = {
     BARCODE_UPNQR,
     BARCODE_ULTRA,
     BARCODE_RMQR,
+    BARCODE_BC412,
     0};
 
 /* ECI TCL encoding names.
  * The ECI comments are given after the name.
  * A ** indicates encodings where native data must be delivered and not utf-8
  */
-static char *s_eci_list[] = {
+static const char *s_eci_list[] = {
     "iso8859-1",    /* 3: ISO-8859-1 - Latin alphabet No. 1 (default)*/
     "iso8859-2",    /* 4: ISO-8859-2 - Latin alphabet No. 2*/
     "iso8859-3",    /* 5: ISO-8859-3 - Latin alphabet No. 3*/
@@ -410,25 +443,32 @@ static char *s_eci_list[] = {
     "cp1251",       /*22: Windows-1251*/
     "cp1252",       /*23: Windows-1252*/
     "cp1256",       /*24: Windows-1256*/
-    "unicode",      /*25: UCS-2BE (High order byte first) Unicode BMP*/
+    "utf-16be",     /*25: UTF-16BE (High order byte first) Unicode*/
     "utf-8",        /*26: Unicode (UTF-8)*/
-    "ascii",        /*27: ISO-646:1991 7-bit character set*/
+    "ascii",        /*27: ISO-646:1991 7-bit character set ASCII*/
     "big5",         /*28: Big5 (Taiwan) Chinese Character Set*/
-    "euc-cn",       /*29: GB (PRC) Chinese Character Set*/
+    "gb2312",       /*29: GB 2312 (PRC) Chinese Character Set*/
     "iso2022-kr",   /*30: Korean Character Set EUC-KR (KS X 1001:2002)*/
+    "gbk",          /*31: GBK Chinese Character Set*/
+    "gb18030",      /*32: GB 18030 Chinese Character Set*/
+    "utf-16le",     /*33: UTF-16LE (Low order byte first) Unicode*/
+    "utf-32be",     /*34: UTF-32BE (High order byte first) Unicode*/
+    "utf-32le",     /*35: UTF-32BE (Low order byte first) Unicode*/
+    "invariant",    /*170: ISO-646:1991 7-bit character set invariant*/
+    "binary",       /*899: 8-bit binary*/
     NULL
 };
 
 /* The ECI numerical number to pass to ZINT */
-static int s_eci_number[] = {
-    3,4,5,6,7,8,9,10,11,12,13,15,16,17,18,20,21,22,23,24,25,26,27,28,29,30
+static const int s_eci_number[] = {
+    3,4,5,6,7,8,9,10,11,12,13,15,16,17,18,20,21,22,23,24,25,26,27,28,29,30,
+    31,32,33,34,35,170,899
 };
 
-
 /* Version information */
-static char version_string[] = VERSION;
+static const char version_string[] = VERSION;
 /* Help text */
-static char help_message[] = "zint tcl(stub,obj) dll\n"
+static const char help_message[] = "zint tcl(stub,obj) dll\n"
     " Generate barcode in tk images\n"
     "Usage:\n"
     " zint encode data photo ?option value? ...\n"
@@ -436,7 +476,7 @@ static char help_message[] = "zint tcl(stub,obj) dll\n"
     "  photo: a tcl photo image handle ('p' after 'image create photo p')\n"
     "  Available options:\n"
     "   -barcode choice: symbology, use 'zint symbology' to get a list\n"
-    "   -addongap number: (7..12, default: 9) set add-on gap in multiple of module size (UPC/EAN-CC)\n"
+    "   -addongap integer: (7..12, default: 9) set add-on gap in multiple of module size (EAN/UPC-CC)\n"
     "   -bg color: set background color as 6 or 8 hex rrggbbaa\n"
     /* cli option --binary internally handled */
     "   -bind bool: bars above/below the code, size set by -border\n"
@@ -444,15 +484,17 @@ static char help_message[] = "zint tcl(stub,obj) dll\n"
     "   -border integer: width of a border around the symbol. Use with -bind/-box 1\n"
     "   -box bool: box around bar code, size set be -border\n"
     /* cli option --cmyk not supported as no corresponding output */
-    "   -cols integer: PDF417, Codablock F, DotCode: number of columns\n"
+    "   -cols integer: Codablock F, DotCode, PDF417: number of columns\n"
+    "   -compliantheight bool: warn if height not compliant, and use standard default\n"
     /* cli option --data is standard parameter */
     "   -dmre bool: Allow Data Matrix Rectangular Extended\n"
     "   -dotsize number: radius ratio of dots from 0.01 to 1.0\n" 
     "   -dotty bool: use dots instead of boxes for matrix codes\n"
     /* cli option --dump not supported */
     /* cli option --ecinos not supported */
-    "   -eci number: ECI to use\n"
+    "   -eci choice: ECI to use\n"
     /* cli option --esc not supported */
+    "   -fast bool: use fast encodation (Data Matrix)\n"
     "   -fg color: set foreground color as 6 or 8 hex rrggbbaa\n"
     /* replaces cli options --binary and --gs1 */
     "   -format binary|unicode|gs1: input data format. Default:unicode\n"
@@ -461,28 +503,35 @@ static char help_message[] = "zint tcl(stub,obj) dll\n"
     "   -gs1nocheck bool: for gs1, do not check validity of data (allows non-standard symbols)\n"
     "   -gs1parens bool: for gs1, AIs enclosed in parentheses instead of square brackets\n"
     "   -gssep bool: for gs1, use gs as separator instead fnc1 (Datamatrix only)\n"
+    "   -guarddescent double: Height of guard bar descent in modules (EAN/UPC only)\n"
     "   -height double: Symbol height in modules\n"
+    "   -heightperrow bool: treat height as per-row\n"
     /* cli option --input not supported */
     "   -init bool: Create reader initialisation symbol (Code 128, Data Matrix)\n"
-    "   -mask number: set masking pattern to use (QR/MicroQR/HanXin/DotCode)\n"
+    "   -mask integer: set masking pattern to use (QR/MicroQR/HanXin/DotCode)\n"
     /* cli option --mirror not supported */
-    "   -mode number: set encoding mode (MaxiCode, Composite)\n"
+    "   -mode integer: set encoding mode (MaxiCode, Composite)\n"
     "   -nobackground bool: set background transparent\n"
+    "   -noquietzones bool: disable default quiet zones\n"
     "   -notext bool: no interpretation line\n"
     /* cli option --output not supported */
     "   -primary text: Structured primary data (MaxiCode, Composite)\n"
+    "   -quietzones bool: add compliant quiet zones to whitespace\n"
     "   -reverse bool: Reverse colours (white on black)\n"
     "   -rotate angle: Image rotation by 0,90 or 270 degrees\n"
-    "   -rows integer: Codablock F: number of rows\n"
+    "   -rows integer: Codablock F, PDF417: number of rows\n"
     "   -scale double: Scale the image to this factor\n"
-    "   -scmvv number: Prefix SCM with [)>\\R01\\Gvv (vv is NUMBER) (MaxiCode)\n"
-    "   -secure integer: EC Level (PDF417, QR)\n"
+    "   -scmvv integer: Prefix SCM with [)>\\R01\\Gvv (vv is integer) (MaxiCode)\n"
+    "   -secure integer: EC Level (Aztec, GridMatrix, HanXin, PDF417, QR, UltraCode)\n"
+    "   -segN {eci data}: Set the ECI & data content for segment N where N is 1 to 9\n"
     "   -separator 0..4 (default: 1) : Stacked symbologies: separator width\n"
     /* cli option --small replaced by -smalltext */
     "   -smalltext bool: tiny interpretation line font\n"
     "   -square bool: force Data Matrix symbols to be square\n"
+    "   -structapp {index count ?id?}: set Structured Append info\n"
     /* cli option --types not supported */
     "   -vers integer: Symbology option\n"
+    /* cli option --version not supported */
     "   -vwhitesp integer: vertical quiet zone in modules\n"
     "   -whitesp integer: horizontal quiet zone in modules\n"
     "   -werror bool: Convert all warnings into errors\n"
@@ -677,7 +726,13 @@ static int Encode(Tcl_Interp *interp, int objc,
     int addon_gap = 0;
     int Separator = 1;
     int Mask = 0;
+    int rows = 0;
     unsigned int cap;
+    int seg_count = 0;
+    int seg_no;
+    Tcl_Obj *pSegDataObjs[10] = {0};
+    Tcl_DString segInputs[10];
+    struct zint_seg segs[10];
     /*------------------------------------------------------------------------*/
     /* >> Check if at least data and object is given and a pair number of */
     /* >> options */
@@ -702,23 +757,29 @@ static int Encode(Tcl_Interp *interp, int objc,
     for (optionPos = 4; optionPos < objc; optionPos+=2) {
         /*--------------------------------------------------------------------*/
         /* Option list and indexes */
-        char *optionList[] = {
+        static const char *optionList[] = {
             "-addongap", "-barcode", "-bg", "-bind", "-bold", "-border", "-box",
-            "-cols", "-dmre", "-dotsize", "-dotty", "-eci", "-fg", "-format",
-            "-fullmultibyte", "-gs1nocheck", "-gs1parens", "-gssep", "-height",
-            "-init", "-mask", "-mode",
-            "-nobackground", "-notext", "-primary", "-reverse", "-rotate",
-            "-rows", "-scale", "-scmvv", "-secure", "-separator", "-smalltext",
-            "-square", "-to", "-vers", "-vwhitesp", "-werror", "-whitesp",
+            "-cols", "-compliantheight", "-dmre", "-dotsize", "-dotty",
+            "-eci", "-fast", "-fg", "-format", "-fullmultibyte",
+            "-gs1nocheck", "-gs1parens", "-gssep", "-guarddescent",
+            "-height", "-heightperrow", "-init", "-mask", "-mode",
+            "-nobackground", "-noquietzones", "-notext", "-primary", "-quietzones",
+            "-reverse", "-rotate", "-rows", "-scale", "-scmvv", "-secure",
+            "-seg1", "-seg2", "-seg3", "-seg4", "-seg5", "-seg6", "-seg7", "-seg8", "-seg9",
+            "-separator", "-smalltext", "-square", "-structapp",
+            "-to", "-vers", "-vwhitesp", "-werror", "-whitesp",
             NULL};
         enum iOption {
             iAddonGap, iBarcode, iBG, iBind, iBold, iBorder, iBox,
-            iCols, iDMRE, iDotSize, iDotty, iECI, iFG, iFormat,
-            iFullMultiByte, iGS1NoCheck, iGS1Parens, iGSSep, iHeight,
-            iInit, iMask, iMode,
-            iNoBackground, iNoText, iPrimary, iReverse, iRotate,
-            iRows, iScale, iSCMvv, iSecure, iSeparator, iSmallText,
-            iSquare, iTo, iVers, iVWhiteSp, iWError, iWhiteSp
+            iCols, iCompliantHeight, iDMRE, iDotSize, iDotty,
+            iECI, iFast, iFG, iFormat, iFullMultiByte,
+            iGS1NoCheck, iGS1Parens, iGSSep, iGuardDescent,
+            iHeight, iHeightPerRow, iInit, iMask, iMode,
+            iNoBackground, iNoQuietZones, iNoText, iPrimary, iQuietZones,
+            iReverse, iRotate, iRows, iScale, iSCMvv, iSecure,
+            iSeg1, iSeg2, iSeg3, iSeg4, iSeg5, iSeg6, iSeg7, iSeg8, iSeg9,
+            iSeparator, iSmallText, iSquare, iStructApp,
+            iTo, iVers, iVWhiteSp, iWError, iWhiteSp
             };
         int optionIndex;
         int intValue;
@@ -726,7 +787,7 @@ static int Encode(Tcl_Interp *interp, int objc,
         /*--------------------------------------------------------------------*/
         if(Tcl_GetIndexFromObj(interp, objv[optionPos],
             (const char **) optionList,
-            "zint option", optionPos-1, &optionIndex)
+            "zint option", 0, &optionIndex)
             == TCL_ERROR)
         {
             fError = 1;
@@ -738,14 +799,19 @@ static int Encode(Tcl_Interp *interp, int objc,
         case iBind:
         case iBold:
         case iBox:
+        case iCompliantHeight:
         case iDMRE:
         case iDotty:
+        case iFast:
         case iGS1NoCheck:
         case iGS1Parens:
         case iGSSep:
+        case iHeightPerRow:
         case iInit:
         case iNoBackground:
+        case iNoQuietZones:
         case iNoText:
+        case iQuietZones:
         case iSmallText:
         case iSquare:
         case iFullMultiByte:
@@ -768,6 +834,8 @@ static int Encode(Tcl_Interp *interp, int objc,
                 fError = 1;
             }
             break;
+        case iHeight:
+        case iGuardDescent:
         case iDotSize:
         case iScale:
             /* >> Float */
@@ -780,7 +848,6 @@ static int Encode(Tcl_Interp *interp, int objc,
         case iAddonGap:
         case iBorder:
         case iCols:
-        case iHeight:
         case iMask:
         case iMode:
         case iRotate:
@@ -806,10 +873,42 @@ static int Encode(Tcl_Interp *interp, int objc,
             Tcl_UtfToExternalDString( hZINTEncoding, pStr, lStr, &dString);
             if (Tcl_DStringLength(&dString) > (optionIndex==iPrimary?90:250)) {
                 Tcl_DStringFree(&dString);
-                Tcl_SetObjResult(interp,Tcl_NewStringObj("String to long", -1));
+                Tcl_SetObjResult(interp,Tcl_NewStringObj("String too long", -1));
                 fError = 1;
             }
             break;
+        case iSeg1: case iSeg2: case iSeg3: case iSeg4: case iSeg5:
+        case iSeg6: case iSeg7: case iSeg8: case iSeg9:
+            seg_no = optionIndex - iSeg1 + 1;
+            if (pSegDataObjs[seg_no]) {
+                Tcl_SetObjResult(interp, Tcl_NewStringObj("duplicate segment", -1));
+                fError = 1;
+            } else {
+                Tcl_Obj *poParam;
+                if (TCL_OK != Tcl_ListObjLength(interp, objv[optionPos+1], &lStr)) {
+                    Tcl_SetObjResult(interp, Tcl_Format(interp, "option %s not a list", 1, objv + optionPos));
+                    fError = 1;
+                } else if (lStr != 2) {
+                    Tcl_SetObjResult(interp, Tcl_Format(interp, "option %s not a list of 2", 1, objv + optionPos));
+                    fError = 1;
+                } else if (TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
+                        0, &poParam)
+                        || TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
+                            1, &pSegDataObjs[seg_no])) {
+                    Tcl_SetObjResult(interp, Tcl_Format(interp, "option %s list format is {eci data}", 1, objv + optionPos));
+                    fError = 1;
+                } else if (Tcl_GetIndexFromObj(interp, poParam,
+                            (const char **) s_eci_list, Tcl_GetString(objv[optionPos]), 0, &ECIIndex)
+                            == TCL_ERROR) {
+                    fError = 1;
+                } else {
+                    segs[seg_no].eci = s_eci_number[ECIIndex];
+                    if (seg_no >= seg_count) {
+                        seg_count = seg_no + 1;
+                    }
+                }
+            }
+            break;
         }
         if (fError) {
             break;
@@ -846,6 +945,13 @@ static int Encode(Tcl_Interp *interp, int objc,
                 my_symbol->output_options &= ~BARCODE_BOX;
             }
             break;
+        case iCompliantHeight:
+            if (intValue) {
+                my_symbol->output_options |= COMPLIANT_HEIGHT;
+            } else {
+                my_symbol->output_options &= ~COMPLIANT_HEIGHT;
+            }
+            break;
         case iDotSize:
             if (doubleValue < 0.01) {
                 Tcl_SetObjResult(interp,
@@ -862,6 +968,13 @@ static int Encode(Tcl_Interp *interp, int objc,
                 my_symbol->output_options &= ~BARCODE_DOTTY_MODE;
             }
             break;
+        case iFast:
+            if (intValue) {
+                my_symbol->input_mode |= FAST_MODE;
+            } else {
+                my_symbol->input_mode &= ~FAST_MODE;
+            }
+            break;
         case iGS1NoCheck:
             if (intValue) {
                 my_symbol->input_mode |= GS1NOCHECK_MODE;
@@ -888,7 +1001,7 @@ static int Encode(Tcl_Interp *interp, int objc,
             break;
         case iECI:
             if(Tcl_GetIndexFromObj(interp, objv[optionPos+1],
-                (const char **) s_eci_list,"-eci", optionPos, &ECIIndex)
+                (const char **) s_eci_list, "-eci", 0, &ECIIndex)
                 == TCL_ERROR)
             {
                 fError = 1;
@@ -896,6 +1009,13 @@ static int Encode(Tcl_Interp *interp, int objc,
                 my_symbol->eci = s_eci_number[ECIIndex];
             }
             break;
+        case iHeightPerRow:
+            if (intValue) {
+                my_symbol->input_mode |= HEIGHTPERROW_MODE;
+            } else {
+                my_symbol->input_mode &= ~HEIGHTPERROW_MODE;
+            }
+            break;
         case iInit:
             if (intValue) {
                 my_symbol->output_options |= READER_INIT;
@@ -924,6 +1044,7 @@ static int Encode(Tcl_Interp *interp, int objc,
         case iFG:
             strncpy(my_symbol->fgcolour, pStr, lStr);
             my_symbol->fgcolour[lStr]='\0';
+            printf("my_symbol->fgcolour %s\n", my_symbol->fgcolour);
             break;
         case iBG:
             strncpy(my_symbol->bgcolour, pStr, lStr);
@@ -934,9 +1055,23 @@ static int Encode(Tcl_Interp *interp, int objc,
                 strcpy(my_symbol->bgcolour, "ffffff00");
             }
             break;
+        case iNoQuietZones:
+            if (intValue) {
+                my_symbol->output_options |= BARCODE_NO_QUIET_ZONES;
+            } else {
+                my_symbol->output_options &= ~BARCODE_NO_QUIET_ZONES;
+            }
+            break;
         case iNoText:
             my_symbol->show_hrt = (intValue?0:1);
             break;
+        case iQuietZones:
+            if (intValue) {
+                my_symbol->output_options |= BARCODE_QUIET_ZONES;
+            } else {
+                my_symbol->output_options &= ~BARCODE_QUIET_ZONES;
+            }
+            break;
         case iSquare:
             /* DM_SQUARE overwrites DM_DMRE */
             if (intValue)
@@ -965,8 +1100,17 @@ static int Encode(Tcl_Interp *interp, int objc,
                 my_symbol->border_width = intValue;
             }
             break;
+        case iGuardDescent:
+            if ((float)doubleValue < 0.0f || (float)doubleValue > 50.0f) {
+                Tcl_SetObjResult(interp,
+                    Tcl_NewStringObj("Guard bar descent out of range", -1));
+                fError = 1;
+            } else {
+                my_symbol->guard_descent = (float)doubleValue;
+            }
+            break;
         case iHeight:
-            if ((float)doubleValue < 0.5f || (float)doubleValue > 1000.0f) {
+            if ((float)doubleValue < 0.5f || (float)doubleValue > 2000.0f) {
                 Tcl_SetObjResult(interp,
                     Tcl_NewStringObj("Height out of range", -1));
                 fError = 1;
@@ -1006,7 +1150,7 @@ static int Encode(Tcl_Interp *interp, int objc,
             /* >> Int in Option 2 */
             if (intValue < 1
                 || (optionIndex==iCols && intValue > 200)
-                || (optionIndex==iVers && intValue > 47))
+                || (optionIndex==iVers && intValue > 999))
             {
                 Tcl_SetObjResult(interp,
                     Tcl_NewStringObj("cols/vers out of range", -1));
@@ -1018,16 +1162,16 @@ static int Encode(Tcl_Interp *interp, int objc,
         case iSecure:
         case iMode:
         case iRows:
-            /* >> Int in Option 1 */
+            /* >> Int in Option 1 for Codablock, Option 3 for PDF417 */
             if ( (optionIndex==iSecure && (intValue < 1 || intValue > 8))
                 || (optionIndex==iMode && (intValue < 0 || intValue > 6))
-                || (optionIndex==iRows && (intValue < 0 || intValue > 44)))
+                || (optionIndex==iRows && (intValue < 0 || intValue > 90)))
             {
                 Tcl_SetObjResult(interp,
                     Tcl_NewStringObj("secure/mode/rows out of range", -1));
                 fError = 1;
             } else {
-                my_symbol->option_1 = intValue;
+                rows = intValue;
             }
             break;
         case iPrimary:
@@ -1043,7 +1187,7 @@ static int Encode(Tcl_Interp *interp, int objc,
                 /*------------------------------------------------------------*/
                 if(Tcl_GetIndexFromObj(interp, objv[optionPos+1],
                     (const char **) rotateList,
-                    "rotate", optionPos, &intValue)
+                    "-rotate", 0, &intValue)
                     == TCL_ERROR)
                 {
                     fError = 1;
@@ -1059,7 +1203,7 @@ static int Encode(Tcl_Interp *interp, int objc,
             break;
         case iBarcode:
             if(Tcl_GetIndexFromObj(interp, objv[optionPos+1],
-                (const char **) s_code_list,"-barcode", optionPos, &intValue)
+                (const char **) s_code_list, "-barcode", 0, &intValue)
                 == TCL_ERROR)
             {
                 fError = 1;
@@ -1073,6 +1217,54 @@ static int Encode(Tcl_Interp *interp, int objc,
         case iWhiteSp:
             my_symbol->whitespace_width = intValue;
             break;
+        case iStructApp:
+            /* >> Decode the -structapp parameter as list of index count ?ID? */
+            {
+                Tcl_Obj *poParam;
+                struct zint_structapp structapp = { 0, 0, "" };
+                char *pStructAppId = NULL;
+                int lStructAppId = 0;
+                if (TCL_OK != Tcl_ListObjLength(interp,
+                    objv[optionPos+1], &lStr))
+                {
+                    fError = 1;
+                } else if ( ! ( lStr == 2 || lStr == 3 ) ) {
+                    Tcl_SetObjResult(interp,
+                        Tcl_NewStringObj(
+                        "option -structapp not a list of 2 or 3", -1));
+                    fError = 1;
+                } else {
+                    if (TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
+                        0, &poParam)
+                        || TCL_OK != Tcl_GetIntFromObj(interp, poParam, &structapp.index)
+                        || TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
+                            1, &poParam)
+                        || TCL_OK != Tcl_GetIntFromObj(interp, poParam, &structapp.count))
+                    {
+                        fError = 1;
+                    }
+                    if (!fError && lStr == 3 && (
+                        TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
+                            2, &poParam)
+                        || !(pStructAppId = Tcl_GetStringFromObj(poParam, &lStructAppId))
+                        || lStructAppId > 32
+                        ))
+                    {
+                        if (lStructAppId > 32) {
+                            Tcl_SetObjResult(interp,
+                                Tcl_NewStringObj("Structured Append ID too long", -1));
+                        }
+                        fError = 1;
+                    }
+                    if (!fError) {
+                        my_symbol->structapp = structapp;
+                        if (lStr == 3 && pStructAppId && lStructAppId) {
+                            strncpy(my_symbol->structapp.id, pStructAppId, lStructAppId);
+                        }
+                    }
+                }
+            }
+            break;
         case iTo:
             /* >> Decode the -to parameter as list of X0 Y0 ?Width Height? */
             {
@@ -1086,25 +1278,29 @@ static int Encode(Tcl_Interp *interp, int objc,
                         Tcl_NewStringObj(
                         "option -to not a list of 2 or 4", -1));
                     fError = 1;
-                } else if ((
-                    TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
-                        0, &poParam)
-                    || TCL_OK != Tcl_GetIntFromObj(interp,poParam,&destX0)
-                    || TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
-                        1, &poParam)
-                    || TCL_OK != Tcl_GetIntFromObj(interp,poParam,&destY0)
-                    || lStr == 4) && (
-                    TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
-                        2, &poParam)
-                    || TCL_OK != Tcl_GetIntFromObj(interp,poParam,
-                        &destWidth)
-                    || TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
-                        3, &poParam)
-                    || TCL_OK != Tcl_GetIntFromObj(interp,poParam,
-                        &destHeight)
-                    ))
-                {
-                    fError = 1;
+                } else {
+                    if (TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
+                            0, &poParam)
+                        || TCL_OK != Tcl_GetIntFromObj(interp,poParam,&destX0)
+                        || TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
+                            1, &poParam)
+                        || TCL_OK != Tcl_GetIntFromObj(interp,poParam,&destY0))
+                    {
+                        fError = 1;
+                    }
+                    if (!fError && lStr == 4 && (
+                        TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
+                            2, &poParam)
+                        || TCL_OK != Tcl_GetIntFromObj(interp,poParam,
+                            &destWidth)
+                        || TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
+                            3, &poParam)
+                        || TCL_OK != Tcl_GetIntFromObj(interp,poParam,
+                            &destHeight)
+                        ))
+                    {
+                        fError = 1;
+                    }
                 }
             }
             break;
@@ -1117,7 +1313,7 @@ static int Encode(Tcl_Interp *interp, int objc,
                 /*------------------------------------------------------------*/
                 if(Tcl_GetIndexFromObj(interp, objv[optionPos+1],
                     (const char **) formatList,
-                    "format", optionPos, &intValue)
+                    "-format", 0, &intValue)
                     == TCL_ERROR)
                 {
                     fError = 1;
@@ -1155,6 +1351,22 @@ static int Encode(Tcl_Interp *interp, int objc,
         my_symbol->option_2 = addon_gap;
     }
     /*------------------------------------------------------------------------*/
+    if (rows) {
+        /* PDF417 and DBAR_EXPSTK use option 3 for rows */
+        if (my_symbol->symbology == BARCODE_PDF417
+                || my_symbol->symbology == BARCODE_PDF417COMP
+                || my_symbol->symbology == BARCODE_HIBC_PDF
+                || my_symbol->symbology == BARCODE_DBAR_EXPSTK
+                || my_symbol->symbology == BARCODE_DBAR_EXPSTK_CC) {
+            my_symbol->option_3 = rows;
+        } else if (my_symbol->symbology == BARCODE_CODABLOCKF
+                || my_symbol->symbology == BARCODE_HIBC_BLOCKF
+                || my_symbol->symbology == BARCODE_CODE16K
+                || my_symbol->symbology == BARCODE_CODE49) {
+            my_symbol->option_1 = rows;
+        }
+    }
+    /*------------------------------------------------------------------------*/
     /* >>> Prepare input dstring and encode it to ECI encoding*/
     Tcl_DStringInit(& dsInput);
     /*------------------------------------------------------------------------*/
@@ -1171,6 +1383,32 @@ static int Encode(Tcl_Interp *interp, int objc,
             pStr = Tcl_DStringValue( &dsInput );
             lStr = Tcl_DStringLength( &dsInput );
         }
+        if (seg_count) {
+            segs[0].source = (unsigned char *) pStr;
+            segs[0].length = lStr;
+            segs[0].eci = my_symbol->eci;
+            for (seg_no = 1; seg_no < seg_count; seg_no++) {
+                if (!pSegDataObjs[seg_no]) {
+                    Tcl_SetObjResult(interp, Tcl_NewStringObj("Segments must be consecutive", -1));
+                    fError = 1;
+                    break;
+                }
+            }
+            if (!fError) {
+                for (seg_no = 1; seg_no < seg_count; seg_no++) {
+                    if ((my_symbol->input_mode & 0x07) == DATA_MODE) {
+                        segs[seg_no].source = (unsigned char *) Tcl_GetByteArrayFromObj(pSegDataObjs[seg_no],
+                            &segs[seg_no].length);
+                    } else {
+                        pStr = Tcl_GetStringFromObj(pSegDataObjs[seg_no], &lStr);
+                        Tcl_DStringInit(& segInputs[seg_no]);
+                        Tcl_UtfToExternalDString( hZINTEncoding, pStr, lStr, &segInputs[seg_no]);
+                        segs[seg_no].source = (unsigned char *) Tcl_DStringValue( &segInputs[seg_no] );
+                        segs[seg_no].length = Tcl_DStringLength( &segInputs[seg_no] );
+                    }
+                }
+            }
+        }
     }
     /*------------------------------------------------------------------------*/
     /* >>> Build symbol graphic */
@@ -1179,8 +1417,13 @@ static int Encode(Tcl_Interp *interp, int objc,
         Tk_PhotoHandle hPhoto;
         /*--------------------------------------------------------------------*/
         /* call zint graphic creation to buffer */
-        ErrorNumber = ZBarcode_Encode_and_Buffer(my_symbol,
-            (unsigned char *) pStr, lStr, rotate_angle);
+        if (seg_count) {
+            ErrorNumber = ZBarcode_Encode_Segs_and_Buffer(my_symbol,
+                segs, seg_count, rotate_angle);
+        } else {
+            ErrorNumber = ZBarcode_Encode_and_Buffer(my_symbol,
+                (unsigned char *) pStr, lStr, rotate_angle);
+        }
         /*--------------------------------------------------------------------*/
         /* >> Show a message */
         if( 0 != ErrorNumber )
@@ -1257,3 +1500,5 @@ static int Encode(Tcl_Interp *interp, int objc,
     }
     return TCL_OK;
 }
+
+/* vim: set ts=4 sw=4 et : */
