@@ -18,6 +18,7 @@
     with this program; if not, write to the Free Software Foundation, Inc.,
     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
+/* SPDX-License-Identifier: GPL-3.0-or-later */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -54,7 +55,7 @@ static void types(void) {
             " 7 C25IND      Industrial 2 of 5        80 DBAR_OMNSTK    GS1 DataBar Stack Omni\n"
             " 8 CODE39      Code 39                  81 DBAR_EXPSTK    GS1 DataBar Exp Stack\n"
             " 9 EXCODE39    Extended Code 39         82 PLANET         USPS PLANET\n"
-            "13 EANX        EAN                      84 MICROPDF417    MicroPDF417\n"
+            "13 EANX        EAN-2 to EAN-13          84 MICROPDF417    MicroPDF417\n"
             "14 EANX_CHK    EAN + Check Digit        85 USPS_IMAIL     USPS Intelligent Mail\n"
             "16 GS1_128     GS1-128                  86 PLESSEY        UK Plessey\n"
             "18 CODABAR     Codabar                  87 TELEPEN_NUM    Telepen Numeric\n"
@@ -80,19 +81,20 @@ static void types(void) {
             "51 PHARMA      Pharmacode One-Track    130 EANX_CC        Composite EAN\n"
             "52 PZN         Pharmazentralnummer     131 GS1_128_CC     Composite GS1-128\n"
             "53 PHARMA_TWO  Pharmacode Two-Track    132 DBAR_OMN_CC    Comp DataBar Omni\n"
-            "55 PDF417      PDF417                  133 DBAR_LTD_CC    Comp DataBar Limited\n"
-            "56 PDF417COMP  Compact PDF417          134 DBAR_EXP_CC    Comp DataBar Expanded\n"
-            "57 MAXICODE    MaxiCode                135 UPCA_CC        Composite UPC-A\n"
-            "58 QRCODE      QR Code                 136 UPCE_CC        Composite UPC-E\n"
-            "60 CODE128B    Code 128 (Subset B)     137 DBAR_STK_CC    Comp DataBar Stacked\n"
-            "63 AUSPOST     AP Standard Customer    138 DBAR_OMNSTK_CC Comp DataBar Stack Omn\n"
-            "66 AUSREPLY    AP Reply Paid           139 DBAR_EXPSTK_CC Comp DataBar Exp Stack\n"
-            "67 AUSROUTE    AP Routing              140 CHANNEL        Channel Code\n"
-            "68 AUSREDIRECT AP Redirection          141 CODEONE        Code One\n"
-            "69 ISBNX       ISBN                    142 GRIDMATRIX     Grid Matrix\n"
-            "70 RM4SCC      Royal Mail 4SCC         143 UPNQR          UPN QR Code\n"
-            "71 DATAMATRIX  Data Matrix             144 ULTRA          Ultracode\n"
-            "72 EAN14       EAN-14                  145 RMQR           Rectangular Micro QR\n"
+            "54 CEPNET      Brazilian CEPNet        133 DBAR_LTD_CC    Comp DataBar Limited\n"
+            "55 PDF417      PDF417                  134 DBAR_EXP_CC    Comp DataBar Expanded\n"
+            "56 PDF417COMP  Compact PDF417          135 UPCA_CC        Composite UPC-A\n"
+            "57 MAXICODE    MaxiCode                136 UPCE_CC        Composite UPC-E\n"
+            "58 QRCODE      QR Code                 137 DBAR_STK_CC    Comp DataBar Stacked\n"
+            "60 CODE128B    Code 128 (Subset B)     138 DBAR_OMNSTK_CC Comp DataBar Stack Omn\n"
+            "63 AUSPOST     AP Standard Customer    139 DBAR_EXPSTK_CC Comp DataBar Exp Stack\n"
+            "66 AUSREPLY    AP Reply Paid           140 CHANNEL        Channel Code\n"
+            "67 AUSROUTE    AP Routing              141 CODEONE        Code One\n"
+            "68 AUSREDIRECT AP Redirection          142 GRIDMATRIX     Grid Matrix\n"
+            "69 ISBNX       ISBN                    143 UPNQR          UPN QR Code\n"
+            "70 RM4SCC      Royal Mail 4SCC         144 ULTRA          Ultracode\n"
+            "71 DATAMATRIX  Data Matrix             145 RMQR           Rectangular Micro QR\n"
+            "72 EAN14       EAN-14                  146 BC412          BC412\n"
             "73 VIN         Vehicle Information No.\n"
             );
 }
@@ -110,7 +112,8 @@ static void version(int no_png) {
         /* This is a test release */
         version_release = version_release / 10;
         version_build = zint_version % 10;
-        printf("Zint version %d.%d.%d.%d (dev)%s\n", version_major, version_minor, version_release, version_build, no_png_lib);
+        printf("Zint version %d.%d.%d.%d (dev)%s\n", version_major, version_minor, version_release, version_build,
+                no_png_lib);
     } else {
         /* This is a stable release */
         printf("Zint version %d.%d.%d%s\n", version_major, version_minor, version_release, no_png_lib);
@@ -126,7 +129,7 @@ static void usage(int no_png) {
 
     printf( "Encode input data in a barcode and save as BMP/EMF/EPS/GIF/PCX%s/SVG/TIF/TXT\n\n"
             "  -b, --barcode=TYPE    Number or name of barcode type. Default is 20 (CODE128)\n"
-            "  --addongap=NUMBER     Set add-on gap in multiples of X-dimension for UPC/EAN\n"
+            "  --addongap=NUMBER     Set add-on gap in multiples of X-dimension for EAN/UPC\n"
             "  --batch               Treat each line of input file as a separate data set\n"
             "  --bg=COLOUR           Specify a background colour (in hex RGB/RGBA)\n"
             "  --binary              Treat input as raw binary data\n"
@@ -154,7 +157,7 @@ static void usage(int no_png) {
             "  --gs1nocheck          Do not check validity of GS1 data\n"
             "  --gs1parens           Process parentheses \"()\" as GS1 AI delimiters, not \"[]\"\n"
             "  --gssep               Use separator GS for GS1 (Data Matrix)\n"
-            "  --guarddescent=NUMBER Set height of guard bar descent in X-dims (UPC/EAN)\n"
+            "  --guarddescent=NUMBER Set height of guard bar descent in X-dims (EAN/UPC)\n"
             "  -h, --help            Display help message\n"
             "  --height=NUMBER       Set height of symbol in multiples of X-dimension\n"
             "  --heightperrow        Treat height as per-row\n"
@@ -292,6 +295,7 @@ static int get_barcode_name(const char *barcode_name) {
         { BARCODE_AZTEC, "azteccode" }, /* Synonym */
         { BARCODE_AZRUNE, "aztecrune" }, /* Synonym */
         { BARCODE_AZRUNE, "aztecrunes" }, /* Synonym */
+        { BARCODE_BC412, "bc412" },
         { BARCODE_C25LOGIC, "c25datalogic" }, /* Synonym */
         { BARCODE_C25IATA, "c25iata" },
         { BARCODE_C25IND, "c25ind" },
@@ -301,6 +305,7 @@ static int get_barcode_name(const char *barcode_name) {
         { BARCODE_C25LOGIC, "c25logic" },
         { BARCODE_C25STANDARD, "c25matrix" },
         { BARCODE_C25STANDARD, "c25standard" },
+        { BARCODE_CEPNET, "cepnet" },
         { BARCODE_CHANNEL, "channel" },
         { BARCODE_CHANNEL, "channelcode" }, /* Synonym */
         { BARCODE_CODABAR, "codabar" },
@@ -448,7 +453,7 @@ static int get_barcode_name(const char *barcode_name) {
         { BARCODE_DBAR_EXPSTK_CC, "rssexpstackcc" }, /* Synonym */
         { BARCODE_DBAR_LTD, "rssltd" }, /* Synonym */
         { BARCODE_DBAR_LTD_CC, "rssltdcc" }, /* Synonym */
-        { BARCODE_C25STANDARD, "standardcode2of5" },
+        { BARCODE_C25STANDARD, "standardcode2of5" }, /* Synonym */
         { BARCODE_TELEPEN, "telepen" },
         { BARCODE_TELEPEN_NUM, "telepennum" },
         { BARCODE_ULTRA, "ultra" },
@@ -675,7 +680,7 @@ static int validate_seg(const char *optarg, const int N, struct zint_seg segs[10
 
 /* Batch mode - output symbol for each line of text in `filename` */
 static int batch_process(struct zint_symbol *symbol, const char *filename, const int mirror_mode,
-            const char *filetype, const int rotate_angle) {
+            const char *filetype, const int output_given, const int rotate_angle) {
     FILE *file;
     unsigned char buffer[ZINT_MAX_DATA_LEN] = {0}; // Maximum HanXin input
     unsigned char character = 0;
@@ -687,7 +692,7 @@ static int batch_process(struct zint_symbol *symbol, const char *filename, const
     int format_len, i, o;
     char adjusted[2] = {0};
 
-    if (symbol->outfile[0] == '\0') {
+    if (symbol->outfile[0] == '\0' || !output_given) {
         strcpy(format_string, "~~~~~.");
         strncat(format_string, filetype, 3);
     } else {
@@ -700,7 +705,8 @@ static int batch_process(struct zint_symbol *symbol, const char *filename, const
     } else {
         file = fopen(filename, "rb");
         if (!file) {
-            sprintf(symbol->errtxt, "102: Unable to read input file '%s'", filename);
+            fprintf(stderr, "Error 102: Unable to read input file '%s'\n", filename);
+            fflush(stderr);
             return ZINT_ERROR_INVALID_DATA;
         }
     }
@@ -763,7 +769,11 @@ static int batch_process(struct zint_symbol *symbol, const char *filename, const
                                 adjusted[0] = reverse_number[inpos - 1];
                                 inpos--;
                             } else {
+#ifndef _WIN32
                                 adjusted[0] = '*';
+#else
+                                adjusted[0] = '+';
+#endif
                             }
                             break;
                         default:
@@ -842,7 +852,10 @@ static int batch_process(struct zint_symbol *symbol, const char *filename, const
             fprintf(stderr, "On line %d: Error 103: Input data too long\n", line_count);
             fflush(stderr);
             do {
-                character = fgetc(file);
+                if ((intChar = fgetc(file)) == EOF) {
+                    break;
+                }
+                character = (unsigned char) intChar;
             } while ((!feof(file)) && (character != '\n'));
         }
     } while ((!feof(file)) && (line_count < 2000000000));
@@ -935,7 +948,7 @@ typedef struct { char *arg; int opt; } arg_opt;
 
 int main(int argc, char **argv) {
     struct zint_symbol *my_symbol;
-    struct zint_seg segs[10] = {0};
+    struct zint_seg segs[10] = {{0}};
     int error_number = 0;
     int warn_number = 0;
     int rotate_angle = 0;
@@ -950,6 +963,7 @@ int main(int argc, char **argv) {
     int addon_gap = 0;
     int rows = 0;
     char filetype[4] = {0};
+    int output_given = 0;
     int no_png;
     int png_refused;
     int val;
@@ -1547,6 +1561,7 @@ int main(int argc, char **argv) {
 
             case 'o':
                 strncpy(my_symbol->outfile, optarg, 255);
+                output_given = 1;
                 break;
 
             case 'r':
@@ -1628,7 +1643,8 @@ int main(int argc, char **argv) {
                 fflush(stderr);
                 warn_number = ZINT_WARN_INVALID_OPTION;
             }
-            error_number = batch_process(my_symbol, arg_opts[0].arg, mirror_mode, filetype, rotate_angle);
+            error_number = batch_process(my_symbol, arg_opts[0].arg, mirror_mode, filetype, output_given,
+                            rotate_angle);
         } else {
             if (seg_count) {
                 if (data_arg_num > 1) {
@@ -1666,7 +1682,8 @@ int main(int argc, char **argv) {
                         ret = ZBarcode_Encode_Segs(my_symbol, segs, seg_count);
                     } else {
                         if (i == 1 && (ZBarcode_Cap(symbology, ZINT_CAP_STACKABLE) & ZINT_CAP_STACKABLE) == 0) {
-                            fprintf(stderr, "Error 173: Symbology must be stackable if multiple data arguments given\n");
+                            fprintf(stderr,
+                                    "Error 173: Symbology must be stackable if multiple data arguments given\n");
                             fflush(stderr);
                             error_number = ZINT_ERROR_INVALID_DATA;
                             break;
