@@ -1,6 +1,6 @@
 /*
     Zint Barcode Generator - the open source barcode generator
-    Copyright (C) 2009 - 2021 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2009-2022 Robin Stuart <rstuart114@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
     with this program; if not, write to the Free Software Foundation, Inc.,
     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
-/* vim: set ts=4 sw=4 et : */
+/* SPDX-License-Identifier: GPL-3.0-or-later */
 
 //#include <QDebug>
 #include <QUiLoader>
@@ -30,7 +30,8 @@
 // Shorthand
 #define QSL QStringLiteral
 
-ExportWindow::ExportWindow(BarcodeItem *bc, const QString& output_data) : m_bc(bc), m_output_data(output_data)
+ExportWindow::ExportWindow(BarcodeItem *bc, const QString& output_data)
+    : m_bc(bc), m_output_data(output_data), m_lines(0)
 {
     setupUi(this);
     QSettings settings;
@@ -44,6 +45,7 @@ ExportWindow::ExportWindow(BarcodeItem *bc, const QString& output_data) : m_bc(b
     linDestPath->setText(settings.value(QSL("studio/export/destination"),
                         QDir::toNativeSeparators(QDir::homePath())).toString());
     linPrefix->setText(settings.value(QSL("studio/export/file_prefix"), QSL("bcs_")).toString());
+    linPostfix->setText(settings.value(QSL("studio/export/file_postfix"), QSL("")).toString());
     cmbFileName->setCurrentIndex(settings.value(QSL("studio/export/name_format"), 0).toInt());
     cmbFileFormat->setCurrentIndex(settings.value(QSL("studio/export/filetype"), 0).toInt());
 
@@ -53,6 +55,14 @@ ExportWindow::ExportWindow(BarcodeItem *bc, const QString& output_data) : m_bc(b
     connect(btnCancel, SIGNAL( clicked( bool )), SLOT(close()));
     connect(btnOK, SIGNAL( clicked( bool )), SLOT(process()));
     connect(btnDestPath, SIGNAL( clicked( bool )), SLOT(get_directory()));
+
+    m_dataList = m_output_data.split('\n');
+    m_lines = m_dataList.size();
+    if (m_lines && m_dataList[m_lines - 1].isEmpty()) {
+        m_lines--;
+    }
+    /*: %1 is number of sequences */
+    lblFeedback->setText(tr("Export Results (%1):").arg(m_lines));
 }
 
 ExportWindow::~ExportWindow()
@@ -65,6 +75,7 @@ ExportWindow::~ExportWindow()
 
     settings.setValue(QSL("studio/export/destination"), linDestPath->text());
     settings.setValue(QSL("studio/export/file_prefix"), linPrefix->text());
+    settings.setValue(QSL("studio/export/file_postfix"), linPostfix->text());
     settings.setValue(QSL("studio/export/name_format"), cmbFileName->currentIndex());
     settings.setValue(QSL("studio/export/filetype"), cmbFileFormat->currentIndex());
 }
@@ -100,18 +111,12 @@ void ExportWindow::process()
     txtFeedback->moveCursor(QTextCursor::End, QTextCursor::MoveAnchor);
     QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
 
-    QStringList dataList = m_output_data.split('\n');
-    int lines = dataList.size();
-    if (lines && dataList[lines - 1].isEmpty()) {
-        lines--;
-    }
-
     QString biggest;
     bool needUrlEscape = false;
 
     const int fileNameIdx = cmbFileName->currentIndex();
     if (fileNameIdx == 1) {
-        biggest = QString::number(lines + 1);
+        biggest = QString::number(m_lines + 1);
     } else {
         needUrlEscape = m_output_data.contains(urlRE);
     }
@@ -139,11 +144,12 @@ void ExportWindow::process()
     }
 
     QString filePathPrefix = linDestPath->text() % QDir::separator() % linPrefix->text();
+    QString postfix = linPostfix->text();
 
     QStringList Feedback;
     int successCount = 0, errorCount = 0;
-    for (int i = 0; i < lines; i++) {
-        const QString &dataString = dataList[i];
+    for (int i = 0; i < m_lines; i++) {
+        const QString &dataString = m_dataList[i];
         QString fileName;
         switch (fileNameIdx) {
             case 0: /* Same as Data (URL Escaped) */
@@ -168,9 +174,9 @@ void ExportWindow::process()
                             default: url_escaped += name_qchar; break;
                         }
                     }
-                    fileName = filePathPrefix % url_escaped % suffix;
+                    fileName = filePathPrefix % url_escaped % postfix % suffix;
                 } else {
-                    fileName = filePathPrefix % dataString % suffix;
+                    fileName = filePathPrefix % dataString % postfix % suffix;
                 }
                 break;
             case 1: { /* Formatted Serial Number */
@@ -180,7 +186,7 @@ void ExportWindow::process()
 
                     pad.fill('0', biggest.length() - this_val.length());
 
-                    fileName = filePathPrefix % pad % this_val % suffix;
+                    fileName = filePathPrefix % pad % this_val % postfix % suffix;
                 }
                 break;
         }
@@ -224,3 +230,5 @@ void ExportWindow::process()
     }
     txtFeedback->moveCursor(QTextCursor::End, QTextCursor::MoveAnchor);
 }
+
+/* vim: set ts=4 sw=4 et : */
