@@ -1,6 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2008 by BogDan Vatra <bogdan@licentia.eu>               *
- *   Copyright (C) 2009-2021 by Robin Stuart <rstuart114@gmail.com>        *
+ *   Copyright (C) 2009-2022 by Robin Stuart <rstuart114@gmail.com>        *
  *                                                                         *
  *   This program is free software: you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -13,10 +13,10 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
-/* vim: set ts=4 sw=4 et : */
+/* SPDX-License-Identifier: GPL-3.0-or-later */
 
-#ifndef MAINWINDOW_H
-#define MAINWINDOW_H
+#ifndef Z_MAINWINDOW_H
+#define Z_MAINWINDOW_H
 
 #include <QtGui>
 #include <QGraphicsItem>
@@ -42,11 +42,23 @@ public:
 
     static QString get_zint_version(void);
 
+#ifdef Q_OS_MACOS
+    static void mac_hack(QWidget *win);
+    static void mac_hack_vLayouts(QWidget *win);
+    static void mac_hack_statusBars(QWidget *win, const char *name = nullptr);
+#endif
+
 public slots:
     void update_preview();
     void change_options();
+
     void on_fgcolor_clicked();
     void on_bgcolor_clicked();
+    void fgcolor_changed(const QColor& color);
+    void bgcolor_changed(const QColor& color);
+    void fgcolor_edited();
+    void bgcolor_edited();
+
     void data_ui_set();
     void composite_ui_set();
     void composite_ean_check();
@@ -61,29 +73,36 @@ public slots:
     void structapp_ui_set();
     void on_encoded();
     void on_errored();
+    void on_dataChanged(const QString& text, bool escaped, int seg_no);
     void filter_symbologies();
 
     bool save();
+    void factory_reset();
     void about();
     void help();
     void quit_now();
     void menu();
 
     void reset_colours();
+    void reverse_colours();
     void open_data_dialog();
     void open_data_dialog_seg1();
     void open_data_dialog_seg2();
     void open_data_dialog_seg3();
     void open_sequence_dialog();
+    void clear_data();
+    void clear_data_seg1();
+    void clear_data_seg2();
+    void clear_data_seg3();
+    void clear_composite();
+    void zap();
     void open_cli_dialog();
 
     void copy_to_clipboard_bmp();
     void copy_to_clipboard_emf();
     void copy_to_clipboard_eps();
     void copy_to_clipboard_gif();
-#ifndef NO_PNG
-    void copy_to_clipboard_png();
-#endif
+    void copy_to_clipboard_png(); // Note Qt can't handle #ifndef NO_PNG in slots
     void copy_to_clipboard_pcx();
     void copy_to_clipboard_svg();
     void copy_to_clipboard_tif();
@@ -100,9 +119,19 @@ public slots:
     void errtxtBar_context_menu(const QPoint &pos);
 
 protected:
-    void resizeEvent(QResizeEvent *event);
-    bool event(QEvent *event) override;
-    bool eventFilter(QObject *watched, QEvent *event);
+    void load_settings(QSettings &settings);
+
+    bool clear_data_eci_seg(int seg_no);
+
+    void color_clicked(QColor &color, QLineEdit *txt, QPushButton *btn, const QString& title, QByteArray& geometry,
+            const char *color_changed);
+    void color_edited(QColor &color, QLineEdit *txt, QPushButton *btn);
+    QString getColorStr(const QColor color, bool alpha_always = false);
+    void setColorTxtBtn(const QColor color, QLineEdit *txt, QPushButton* btn);
+
+    virtual void resizeEvent(QResizeEvent *event) override;
+    virtual bool event(QEvent *event) override;
+    virtual bool eventFilter(QObject *watched, QEvent *event) override;
 
     void combobox_item_enabled(QComboBox *comboBox, int index, bool enabled);
     void upcean_addon_gap(const QString &comboBoxName, const QString &labelName, int base);
@@ -115,12 +144,17 @@ protected:
 
     void createActions();
     void createMenu();
-    void enableActions(bool enabled);
+    void enableActions();
 
     void copy_to_clipboard(const QString &filename, const QString &name, const char *mimeType = nullptr);
 
     void errtxtBar_clear();
-    void errtxtBar_set(bool isError);
+    void errtxtBar_set();
+
+    void automatic_info_set();
+
+    QLineEdit *get_seg_textbox(int seg_no);
+    QComboBox *get_seg_eci(int seg_no);
 
     QPoint get_context_menu_pos(const QPoint &pos, QWidget *widget);
 
@@ -154,13 +188,15 @@ protected:
     void load_sub_settings(QSettings &settings, int symbology);
 
 private:
-    QColor m_fgcolor,m_bgcolor;
+    QColor m_fgcolor, m_bgcolor;
+    QByteArray m_fgcolor_geometry, m_bgcolor_geometry;
     BarcodeItem m_bc;
     QWidget *m_optionWidget;
     QGraphicsScene *scene;
     int m_symbology;
     QMenu *m_menu;
     QShortcut *m_saveAsShortcut;
+    QShortcut *m_factoryResetShortcut;
     QShortcut *m_openCLIShortcut;
     QShortcut *m_copyBMPShortcut;
     QShortcut *m_copyEMFShortcut;
@@ -180,6 +216,7 @@ private:
     QAction *m_copyTIFAct;
     QAction *m_openCLIAct;
     QAction *m_saveAsAct;
+    QAction *m_factoryResetAct;
     QAction *m_aboutAct;
     QAction *m_helpAct;
     QAction *m_quitAct;
@@ -190,4 +227,5 @@ private:
     QPushButton *m_btnHeightPerRowDefault;
 };
 
+/* vim: set ts=4 sw=4 et : */
 #endif
